/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client;

import javax.swing.UIManager;

import net.sf.distriblend.client.gui.Client;
import net.sf.distriblend.share.Settings;

/**
 * The <code>Main</code> class is the entry point for the client application.
 * 
 * <p>
 * The client application is a user-friendly tool to form projects from a set of
 * files and to upload them to the distributor. A project also contains
 * instructions on how to render the files - such as the start and end frames
 * and the image format to produce.
 * </p>
 * 
 * <p>
 * This application also allows the user to check the progress of their projects
 * and when completed, download the rendered images.
 * </p>
 * 
 * @author Ian Thompson
 */
public class Main {

	/**
	 * The entry point method for the client application.
	 * 
	 * @param args
	 *            command line arguments.
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.out.println("Failed to set GUI look and feel.");
		}

		Settings settings = Settings.instance();
		settings.setIntProperty("connect.distributor.port", 55100);
		settings.setIntProperty("connect.distributor.timeout", 30000);

		new Main();
	}

	/**
	 * Constructs the client application by creating a GUI object.
	 */
	public Main() {
		new Client();
	}
}
