/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.FocusEvent;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.WindowConstants;

import net.sf.distriblend.client.gui.RangesListModel.Range;
import net.sf.distriblend.share.ImageFormat;
import net.sf.distriblend.share.LocalPath;
import net.sf.distriblend.share.Task;

/**
 * @author Ian Thompson
 * 
 */
public class AddTaskDialog {

	/**
	 * 
	 */
	public static final int OK_OPTION = 0;

	/**
	 * 
	 */
	public static final int CANCEL_OPTION = 1;

	/**
	 * @param sourceFile
	 * @return Task
	 */
	public static Task showAddTaskDialog(LocalPath sourceFile) {

		AddTaskDialog atd = new AddTaskDialog();
		atd.getSourceFileField().setText(sourceFile.getName());
		atd.getDialog().setVisible(true);

		if (atd.selectedOption == OK_OPTION) {
			ImageFormat format = (ImageFormat) atd.getImageFormatComboBox()
					.getSelectedItem();
			if (format == null)
				return null;
			Task task = new Task(sourceFile, format);
			Iterator<Range> ranges = atd.getFramesListModel().ranges();
			while (ranges.hasNext()) {
				Range r = ranges.next();
				task.addRange(r.lower, r.upper, r.step);
			}
			return task;
		}
		return null;
	}

	private int selectedOption = CANCEL_OPTION;

	private JDialog dialog = null; // @jve:decl-index=0:visual-constraint="10,50"

	private JPanel contentPane = null;

	private JPanel mainPanel = null;

	private JLabel sourceFileLabel = null;

	private JLabel imageFormatLabel = null;

	private JComboBox imageFormatComboBox = null;

	private JList framesList = null;

	private RangesListModel framesListModel = null;

	private JScrollPane framesScrollPane = null;

	private JTextField firstFrameTextField = null;

	private JTextField lastFrameTextField = null;

	private JTextField sourceFileField = null;

	private JButton addRangeButton = null;

	private JButton removeRangeButton = null;

	private JPanel buttonsPanel = null;

	private JButton okButton = null;

	private JButton cancelButton = null;

	private JTextPane dialogInfoTextPane = null;

	private JLabel toLabel = null;

	private JLabel stepLabel = null;

	private JTextField stepTextField = null;

	private JPanel framesPanel = null;

	private JTextPane dialogInfoTextPane2 = null;

	/**
	 * This method initializes stepTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getStepTextField() {
		if (stepTextField == null) {
			stepTextField = new JTextField();
			stepTextField.setText("1");
		}
		return stepTextField;
	}

	/**
	 * This method initializes dialog
	 * 
	 * @return javax.swing.JDialog
	 */
	private JDialog getDialog() {
		if (dialog == null) {
			dialog = new JDialog();
			dialog.setSize(new Dimension(366, 372));
			dialog.setTitle("Add Task");
			dialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			dialog.setModal(true);
			dialog.setIconImage(Icons.ADD_TASK_DIALOG_ICON.getImage());
			dialog.setContentPane(getContentPane());
			dialog.getRootPane().setDefaultButton(getOkButton());
		}
		return dialog;
	}

	/**
	 * This method initializes contentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getContentPane() {
		if (contentPane == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.ipadx = 0;
			gridBagConstraints11.ipady = 0;
			gridBagConstraints11.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints11.weightx = 1.0;
			gridBagConstraints11.weighty = 1.0;
			gridBagConstraints11.fill = GridBagConstraints.BOTH;
			gridBagConstraints11.gridy = 1;
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.gridx = 0;
			gridBagConstraints9.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints9.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints9.weightx = 1.0;
			gridBagConstraints9.gridy = 2;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 0;
			gridBagConstraints7.ipadx = 0;
			gridBagConstraints7.ipady = 0;
			gridBagConstraints7.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints7.weightx = 1.0;
			gridBagConstraints7.weighty = 0.0;
			gridBagConstraints7.fill = GridBagConstraints.BOTH;
			gridBagConstraints7.gridy = 0;
			contentPane = new JPanel();
			contentPane.setLayout(new GridBagLayout());
			contentPane.add(getMainPanel(), gridBagConstraints7);
			contentPane.add(getButtonsPanel(), gridBagConstraints9);
			contentPane.add(getFramesPanel(), gridBagConstraints11);
		}
		return contentPane;
	}

	/**
	 * This method initializes mainPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getMainPanel() {
		if (mainPanel == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints11.gridwidth = 3;
			gridBagConstraints11.fill = GridBagConstraints.BOTH;
			gridBagConstraints11.weighty = 1.0;
			gridBagConstraints11.weightx = 1.0;
			gridBagConstraints11.gridy = 1;
			stepLabel = new JLabel();
			stepLabel.setText("step");
			toLabel = new JLabel();
			toLabel.setText("to");
			GridBagConstraints gridBagConstraints81 = new GridBagConstraints();
			gridBagConstraints81.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints81.gridy = 0;
			gridBagConstraints81.weightx = 1.0;
			gridBagConstraints81.weighty = 0.0;
			gridBagConstraints81.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints81.gridwidth = 7;
			gridBagConstraints81.gridx = 0;
			GridBagConstraints gridBagConstraints41 = new GridBagConstraints();
			gridBagConstraints41.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints41.gridy = 1;
			gridBagConstraints41.weightx = 1.0;
			gridBagConstraints41.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints41.gridwidth = 2;
			gridBagConstraints41.gridx = 1;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints3.gridy = 2;
			gridBagConstraints3.weightx = 1.0;
			gridBagConstraints3.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints3.gridwidth = 2;
			gridBagConstraints3.gridx = 1;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints2.anchor = GridBagConstraints.WEST;
			gridBagConstraints2.gridy = 2;
			imageFormatLabel = new JLabel();
			imageFormatLabel.setText("Image format:");
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints.anchor = GridBagConstraints.WEST;
			gridBagConstraints.gridy = 1;
			sourceFileLabel = new JLabel();
			sourceFileLabel.setText("Source file:");
			mainPanel = new JPanel();
			mainPanel.setLayout(new GridBagLayout());
			mainPanel.setBorder(BorderFactory
					.createTitledBorder("Project Task"));
			mainPanel.add(sourceFileLabel, gridBagConstraints);
			mainPanel.add(imageFormatLabel, gridBagConstraints2);
			mainPanel.add(getImageFormatComboBox(), gridBagConstraints3);
			mainPanel.add(getSourceFileField(), gridBagConstraints41);
			mainPanel.add(getDialogInfoTextPane(), gridBagConstraints81);
		}
		return mainPanel;
	}

	/**
	 * This method initializes imageFormatComboBox
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getImageFormatComboBox() {
		if (imageFormatComboBox == null) {
			imageFormatComboBox = new JComboBox();
			imageFormatComboBox.setModel(new DefaultComboBoxModel(ImageFormat
					.values()));
			imageFormatComboBox
					.addItemListener(new java.awt.event.ItemListener() {
						public void itemStateChanged(java.awt.event.ItemEvent e) {
							validateForm();
						}
					});
		}
		return imageFormatComboBox;
	}

	/**
	 * This method initializes framesList
	 * 
	 * @return javax.swing.JList
	 */
	private JList getFramesList() {
		if (framesList == null) {
			framesList = new JList(getFramesListModel());
			framesList.setCellRenderer(new IconedRenderers.SingleIconListCell(
					Icons.IMAGE_FORMAT.getImageIcon()));
			framesList
					.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
						public void valueChanged(
								javax.swing.event.ListSelectionEvent e) {
							if (framesList.getSelectedIndex() == -1) {
								getRemoveRangeButton().setEnabled(false);
							} else {
								getRemoveRangeButton().setEnabled(true);
							}
						}
					});
		}
		return framesList;
	}

	/**
	 * 
	 * @return RangesListModel
	 */
	private RangesListModel getFramesListModel() {
		if (framesListModel == null) {
			framesListModel = new RangesListModel();
		}
		return framesListModel;
	}

	/**
	 * This method initializes framesScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getFramesScrollPane() {
		if (framesScrollPane == null) {
			framesScrollPane = new JScrollPane();
			framesScrollPane.setViewportView(getFramesList());
		}
		return framesScrollPane;
	}

	/**
	 * This method initializes firstFrameTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getFirstFrameTextField() {
		if (firstFrameTextField == null) {
			firstFrameTextField = new JTextField();
			firstFrameTextField
					.addFocusListener(new java.awt.event.FocusAdapter() {
						@Override
						public void focusGained(java.awt.event.FocusEvent e) {
							getDialog().getRootPane().setDefaultButton(
									getAddRangeButton());
						}

						@Override
						public void focusLost(FocusEvent e) {
							getDialog().getRootPane().setDefaultButton(
									getOkButton());
						}
					});
		}
		return firstFrameTextField;
	}

	/**
	 * This method initializes lastFrameTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getLastFrameTextField() {
		if (lastFrameTextField == null) {
			lastFrameTextField = new JTextField();
			lastFrameTextField
					.addFocusListener(new java.awt.event.FocusAdapter() {
						@Override
						public void focusGained(java.awt.event.FocusEvent e) {
							getDialog().getRootPane().setDefaultButton(
									getAddRangeButton());
						}

						@Override
						public void focusLost(FocusEvent e) {
							getDialog().getRootPane().setDefaultButton(
									getOkButton());
						}
					});
		}
		return lastFrameTextField;
	}

	/**
	 * This method initializes sourceFileField
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JTextField getSourceFileField() {
		if (sourceFileField == null) {
			sourceFileField = new JTextField();
			sourceFileField.setEditable(false);
		}
		return sourceFileField;
	}

	/**
	 * This method initializes addRangeButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getAddRangeButton() {
		if (addRangeButton == null) {
			addRangeButton = new JButton();
			addRangeButton.setText("Add range");
			addRangeButton.addFocusListener(new java.awt.event.FocusAdapter() {
				@Override
				public void focusGained(java.awt.event.FocusEvent e) {
					getDialog().getRootPane().setDefaultButton(
							getAddRangeButton());
				}

				@Override
				public void focusLost(FocusEvent e) {
					getDialog().getRootPane().setDefaultButton(getOkButton());
				}
			});
			addRangeButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							boolean ok = true;
							int first = 0, last = 0, step = 0;
							try {
								first = Integer
										.parseInt(getFirstFrameTextField()
												.getText());
								last = Integer.parseInt(getLastFrameTextField()
										.getText());
								step = Integer.parseInt(getStepTextField()
										.getText());
							} catch (NumberFormatException nfe) {
								ok = false;
							}

							if (ok && first <= last && step > 0) {
								getFramesListModel()
										.addRange(first, last, step);
								getFirstFrameTextField().setText("");
								getLastFrameTextField().setText("");
								getStepTextField().setText("1");
								validateForm();
							} else {
								JOptionPane
										.showMessageDialog(
												getDialog(),
												"A frame range may only contain whole numbers with step greater than zero.",
												"Add Frame Range",
												JOptionPane.INFORMATION_MESSAGE,
												Icons.ADD_TASK_DIALOG_ICON
														.getImageIcon());
							}
						}
					});
		}
		return addRangeButton;
	}

	/**
	 * This method initializes removeRangeButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getRemoveRangeButton() {
		if (removeRangeButton == null) {
			removeRangeButton = new JButton();
			removeRangeButton.setText("Remove range");
			removeRangeButton.setEnabled(false);
			removeRangeButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							int i = getFramesList().getSelectedIndex();
							if (i != -1) {
								getFramesListModel().remove(i);
								validateForm();
							}
						}
					});
		}
		return removeRangeButton;
	}

	/**
	 * This method initializes buttonsPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getButtonsPanel() {
		if (buttonsPanel == null) {
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.gridx = 1;
			gridBagConstraints8.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints8.gridy = 0;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.anchor = GridBagConstraints.EAST;
			buttonsPanel = new JPanel();
			buttonsPanel.setLayout(new GridBagLayout());
			buttonsPanel.add(getOkButton(), gridBagConstraints1);
			buttonsPanel.add(getCancelButton(), gridBagConstraints8);
		}
		return buttonsPanel;
	}

	/**
	 * This method initializes okButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText("OK");
			okButton.setEnabled(false);
			okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					selectedOption = OK_OPTION;
					getDialog().dispose();
				}
			});
		}
		return okButton;
	}

	/**
	 * This method initializes cancelButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText("Cancel");
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					selectedOption = CANCEL_OPTION;
					getDialog().dispose();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * This method initializes dialogInfoTextPane
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getDialogInfoTextPane() {
		if (dialogInfoTextPane == null) {
			dialogInfoTextPane = new JTextPane();
			dialogInfoTextPane.setEnabled(false);
			dialogInfoTextPane.setDisabledTextColor(dialogInfoTextPane
					.getForeground());
			dialogInfoTextPane
					.setText("Select the format for the rendered frames.");
			dialogInfoTextPane.setEditable(false);
			dialogInfoTextPane.setOpaque(false);
		}
		return dialogInfoTextPane;
	}

	/* -- Validation -- */

	/**
	 * 
	 */
	private void validateForm() {
		getOkButton().setEnabled(isFormValid());
	}

	private boolean isFormValid() {
		if (getImageFormatComboBox().getSelectedIndex() == -1)
			return false;
		if (getFramesListModel().getSize() == 0)
			return false;
		return true;
	}

	/**
	 * This method initializes framesPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getFramesPanel() {
		if (framesPanel == null) {
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.fill = GridBagConstraints.BOTH;
			gridBagConstraints4.gridy = 0;
			gridBagConstraints4.weightx = 1.0;
			gridBagConstraints4.weighty = 0.0;
			gridBagConstraints4.gridwidth = 6;
			gridBagConstraints4.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints4.gridx = 0;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.anchor = GridBagConstraints.SOUTH;
			gridBagConstraints6.gridx = 5;
			gridBagConstraints6.gridy = 1;
			gridBagConstraints6.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints6.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.fill = GridBagConstraints.BOTH;
			gridBagConstraints5.gridwidth = 5;
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.gridy = 1;
			gridBagConstraints5.weightx = 1.0;
			gridBagConstraints5.weighty = 1.0;
			gridBagConstraints5.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints16 = new GridBagConstraints();
			gridBagConstraints16.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints16.gridx = 5;
			gridBagConstraints16.gridy = 2;
			gridBagConstraints16.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints15 = new GridBagConstraints();
			gridBagConstraints15.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints15.gridx = 4;
			gridBagConstraints15.gridy = 2;
			gridBagConstraints15.weightx = 1.0;
			gridBagConstraints15.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			gridBagConstraints13.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints13.gridy = 2;
			gridBagConstraints13.gridx = 3;
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			gridBagConstraints12.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints12.gridy = 2;
			gridBagConstraints12.gridx = 1;
			GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
			gridBagConstraints14.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints14.gridx = 0;
			gridBagConstraints14.gridy = 2;
			gridBagConstraints14.weightx = 1.0;
			gridBagConstraints14.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints10.gridx = 2;
			gridBagConstraints10.gridy = 2;
			gridBagConstraints10.weightx = 1.0;
			gridBagConstraints10.insets = new Insets(4, 4, 4, 4);
			framesPanel = new JPanel();
			framesPanel.setLayout(new GridBagLayout());
			framesPanel.setBorder(BorderFactory.createTitledBorder("Frames"));
			framesPanel.add(getLastFrameTextField(), gridBagConstraints10);
			framesPanel.add(getFirstFrameTextField(), gridBagConstraints14);
			framesPanel.add(toLabel, gridBagConstraints12);
			framesPanel.add(stepLabel, gridBagConstraints13);
			framesPanel.add(getStepTextField(), gridBagConstraints15);
			framesPanel.add(getAddRangeButton(), gridBagConstraints16);
			framesPanel.add(getFramesScrollPane(), gridBagConstraints5);
			framesPanel.add(getRemoveRangeButton(), gridBagConstraints6);
			framesPanel.add(getDialogInfoTextPane2(), gridBagConstraints4);
		}
		return framesPanel;
	}

	/**
	 * This method initializes dialogInfoTextPane2
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getDialogInfoTextPane2() {
		if (dialogInfoTextPane2 == null) {
			dialogInfoTextPane2 = new JTextPane();
			dialogInfoTextPane2.setEnabled(false);
			dialogInfoTextPane2
					.setText("Add frame ranges or single frames as required.");
			dialogInfoTextPane2.setDisabledTextColor(getDialogInfoTextPane2()
					.getForeground());
			dialogInfoTextPane2.setEditable(false);
			dialogInfoTextPane2.setOpaque(false);
		}
		return dialogInfoTextPane2;
	}

}
