/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.WindowConstants;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreePath;

import net.sf.distriblend.share.FileMap;
import net.sf.distriblend.share.LocalPath;
import net.sf.distriblend.share.SafeString;
import net.sf.distriblend.share.Task;
import net.sf.distriblend.share.net.AuthorizedConnector;
import net.sf.distriblend.share.net.event.ConnectionEvent;
import net.sf.distriblend.share.net.event.ConnectionListener;
import net.sf.distriblend.share.protocol.Submission;
import net.sf.distriblend.share.protocol.Submission.DeleteResponse;
import net.sf.distriblend.share.protocol.Submission.MonitorItem;
import net.sf.distriblend.share.protocol.Submission.MonitorTasksRequest;
import net.sf.distriblend.share.protocol.Submission.MonitorTasksResponse;

/**
 * 
 * @author Ian Thompson
 * 
 */
public class Client {

	/**
	 * Creates a new client GUI and displays it.
	 * 
	 */
	public Client() {
		getFrame().setVisible(true);
		worker = new Worker();
		monitoring = new Monitoring();
	}

	/* -- Private GUI -- */

	private final Worker worker;

	private final Monitoring monitoring;

	private JFrame frame = null; // @jve:decl-index=0:visual-constraint="10,10"

	private JPanel contentPane = null;

	private BusyPane busyPane = null;

	private JTabbedPane tabbedPane = null;

	private JPanel connectTab = null;

	private JTextPane clientDescriptionText = null;

	private JPanel monitorProjectsTab = null;

	private JPanel createProjectTab = null;

	private JPanel filesTasksBox = null;

	private JPanel monitorProjectsBox = null;

	private JPanel projectBox = null;

	private JScrollPane monitorListScrollPane = null;

	private JTree projectFilesTree = null;

	private ProjectTreeModel projectFilesTreeModel = null;

	private JScrollPane projectFilesScrollPane = null;

	private JButton addFilesButton = null;

	private JPanel newConnectionBox = null;

	private JPanel disconnectedCard = null;

	private JLabel addressLabel = null;

	private JTextField addressTextField = null;

	private JButton connectButton = null;

	private JTextPane makeConnectionDescriptionText = null;

	private JPanel connectedCard = null;

	private JLabel connectedLabel = null;

	private JButton disconnectButton = null;

	private JLabel userLabel = null;

	private JLabel userValueLabel = null;

	private JPanel welcomeBox = null;

	private JPanel savedConnectionsBox = null;

	private JScrollPane connectionsScrollPane = null;

	private ConnectionsList connectionsList = null;

	private JTextPane connectionsDescriptionText = null;

	private JButton storeConnectionButton = null;

	private JTextPane monitoringDescriptionText = null;

	private JButton removeConnectionButton = null;

	private JButton editConnectionButton = null;

	private JTextField projectNameField = null;

	private JLabel projectLabel = null;

	private JScrollPane tasksScrollPane = null;

	private JPanel projectSubmissionBox = null;

	private JButton submitButton = null;

	private JTextPane managerDescriptionText = null;

	private JTextPane projectDescriptionText = null;

	private MonitorList monitorList = null;

	private JButton deleteProjectButton = null;

	private JTextPane submissionDescriptionText = null;

	private JList tasksList = null;

	private DefaultListModel tasksListModel = null;

	private JButton addTaskButton = null;

	private JButton removeTaskButton = null;

	private JLabel localFilesLabel = null;

	private JLabel trafficIndicator = null;

	private JFileChooser archiveFileChooser = null;

	/**
	 * This method initializes frame
	 * 
	 * @return javax.swing.JFrame
	 */
	private JFrame getFrame() {
		if (frame == null) {
			frame = new JFrame();
			frame.setSize(new Dimension(501, 480));
			frame.setTitle("Distriblend Client");
			frame.setIconImage(Icons.CLIENT_ICON.getImage());
			frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			frame.setContentPane(getContentPane());
			frame.setGlassPane(getBusyPane());
		}
		return frame;
	}

	/**
	 * 
	 * @return Busy glass pane component
	 */
	private BusyPane getBusyPane() {
		if (busyPane == null) {
			busyPane = new BusyPane("Busy...");
		}
		return busyPane;
	}

	/**
	 * This method initializes contentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getContentPane() {
		if (contentPane == null) {
			contentPane = new JPanel();
			contentPane.setLayout(new BorderLayout());
			contentPane.add(getTabbedPane(), BorderLayout.CENTER);
		}
		return contentPane;
	}

	/**
	 * This method initializes tabbedPane
	 * 
	 * @return javax.swing.JTabbedPane
	 */
	private JTabbedPane getTabbedPane() {
		if (tabbedPane == null) {
			tabbedPane = new JTabbedPane();
			tabbedPane.addTab("Connect", Icons.TAB_CONNECT.getImageIcon(),
					getConnectTab(), null);
			tabbedPane.addTab("Create Project", Icons.TAB_EDIT.getImageIcon(),
					getCreateProjectTab(), null);
			tabbedPane.addTab("Monitor Projects", Icons.TAB_MONITOR
					.getImageIcon(), getMonitorProjectsTab(), null);
			tabbedPane
					.addChangeListener(new javax.swing.event.ChangeListener() {
						public void stateChanged(javax.swing.event.ChangeEvent e) {
							int index = tabbedPane.getSelectedIndex();
							if (index != 0 && !worker.isConnected()) {
								getBusyPane().setText("Disconnected.");
								getBusyPane().setVisible(true);
								JOptionPane
										.showMessageDialog(
												getFrame(),
												"You must log in to a Distributor to edit or monitor your projects.",
												"Connect",
												JOptionPane.INFORMATION_MESSAGE,
												Icons.CONNECT_DIALOG_ICON
														.getImageIcon());
								tabbedPane.setSelectedIndex(0);
								getBusyPane().setVisible(false);
							}

							/* Monitoring panel activated / deactivated. */
							if (index == 2)
								monitoring.enable();
							else
								monitoring.disable();
						}
					});
		}
		return tabbedPane;
	}

	/* -- Connect Tab -- */

	/**
	 * This method initializes connectTab
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getConnectTab() {
		if (connectTab == null) {
			GridBagConstraints gridBagConstraints23 = new GridBagConstraints();
			gridBagConstraints23.gridx = 0;
			gridBagConstraints23.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints23.fill = GridBagConstraints.BOTH;
			gridBagConstraints23.weightx = 1.0;
			gridBagConstraints23.weighty = 1.0;
			gridBagConstraints23.anchor = GridBagConstraints.CENTER;
			gridBagConstraints23.gridy = 1;
			GridBagConstraints gridBagConstraints22 = new GridBagConstraints();
			gridBagConstraints22.gridx = 0;
			gridBagConstraints22.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints22.fill = GridBagConstraints.BOTH;
			gridBagConstraints22.weightx = 1.0;
			gridBagConstraints22.weighty = 0.0;
			gridBagConstraints22.gridy = 0;
			GridBagConstraints gridBagConstraints110 = new GridBagConstraints();
			gridBagConstraints110.gridx = 0;
			gridBagConstraints110.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints110.fill = GridBagConstraints.BOTH;
			gridBagConstraints110.weightx = 1.0;
			gridBagConstraints110.gridy = 2;
			connectTab = new JPanel();
			connectTab.setLayout(new GridBagLayout());
			connectTab.add(getNewConnectionBox(), gridBagConstraints110);
			connectTab.add(getWelcomeBox(), gridBagConstraints22);
			connectTab.add(getSavedConnectionsBox(), gridBagConstraints23);
		}
		return connectTab;
	}

	/**
	 * This method initializes welcomeBox
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getWelcomeBox() {
		if (welcomeBox == null) {
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.BOTH;
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 1;
			gridBagConstraints.weightx = 1.0;
			gridBagConstraints.weighty = 1.0;
			gridBagConstraints.insets = new Insets(4, 4, 4, 4);
			welcomeBox = new JPanel();
			welcomeBox.setLayout(new GridBagLayout());
			welcomeBox.setBorder(BorderFactory
					.createTitledBorder("Distriblend Client"));
			welcomeBox.add(getClientDescriptionText(), gridBagConstraints);
		}
		return welcomeBox;
	}

	/**
	 * This method initializes clientDescriptionText
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getClientDescriptionText() {
		if (clientDescriptionText == null) {
			clientDescriptionText = new JTextPane();
			clientDescriptionText
					.setText("This is the Distriblend client application for communication with a Distriblend distributor. Use the tabbed panels to organize your projects and submit them for rendering. This panel allows you to form the connection to a distributor.");
			clientDescriptionText.setOpaque(false);
			clientDescriptionText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			clientDescriptionText.setDisabledTextColor(clientDescriptionText
					.getForeground());
			clientDescriptionText.setEnabled(false);
			clientDescriptionText.setEditable(false);
		}
		return clientDescriptionText;
	}

	/**
	 * This method initializes savedConnectionsBox
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getSavedConnectionsBox() {
		if (savedConnectionsBox == null) {
			GridBagConstraints gridBagConstraints28 = new GridBagConstraints();
			gridBagConstraints28.gridx = 0;
			gridBagConstraints28.weightx = 1.0;
			gridBagConstraints28.anchor = GridBagConstraints.EAST;
			gridBagConstraints28.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints28.gridy = 2;
			GridBagConstraints gridBagConstraints27 = new GridBagConstraints();
			gridBagConstraints27.gridx = 1;
			gridBagConstraints27.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints27.anchor = GridBagConstraints.EAST;
			gridBagConstraints27.gridy = 2;
			GridBagConstraints gridBagConstraints24 = new GridBagConstraints();
			gridBagConstraints24.fill = GridBagConstraints.BOTH;
			gridBagConstraints24.gridy = 0;
			gridBagConstraints24.weightx = 1.0;
			gridBagConstraints24.weighty = 0.0;
			gridBagConstraints24.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints24.gridwidth = 2;
			gridBagConstraints24.gridx = 0;
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.fill = GridBagConstraints.BOTH;
			gridBagConstraints8.gridy = 1;
			gridBagConstraints8.weightx = 1.0;
			gridBagConstraints8.weighty = 1.0;
			gridBagConstraints8.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints8.gridwidth = 2;
			gridBagConstraints8.gridx = 0;
			savedConnectionsBox = new JPanel();
			savedConnectionsBox.setLayout(new GridBagLayout());
			savedConnectionsBox.setBorder(BorderFactory
					.createTitledBorder("Stored Connections"));
			savedConnectionsBox.add(getConnectionsScrollPane(),
					gridBagConstraints8);
			savedConnectionsBox.add(getConnectionsDescriptionText(),
					gridBagConstraints24);
			savedConnectionsBox.add(getRemoveConnectionButton(),
					gridBagConstraints27);
			savedConnectionsBox.add(getEditConnectionButton(),
					gridBagConstraints28);
		}
		return savedConnectionsBox;
	}

	/**
	 * This method initializes connectionsDescriptionText
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getConnectionsDescriptionText() {
		if (connectionsDescriptionText == null) {
			connectionsDescriptionText = new JTextPane();
			connectionsDescriptionText.setOpaque(false);
			connectionsDescriptionText.setEditable(false);
			connectionsDescriptionText.setEnabled(false);
			connectionsDescriptionText
					.setDisabledTextColor(connectionsDescriptionText
							.getForeground());
			connectionsDescriptionText
					.setText("Select a previous connection from the list or enter a new address below, then click Connect.");
		}
		return connectionsDescriptionText;
	}

	/**
	 * This method initializes connectionsScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getConnectionsScrollPane() {
		if (connectionsScrollPane == null) {
			connectionsScrollPane = new JScrollPane();
			connectionsScrollPane.setViewportView(getConnectionsList());
		}
		return connectionsScrollPane;
	}

	/**
	 * This method initializes connectionsList
	 * 
	 * @return javax.swing.JList
	 */
	private ConnectionsList getConnectionsList() {
		if (connectionsList == null) {
			File clientDir = new File(System.getProperty("user.home"),
					".distriblend");
			clientDir.mkdir();
			connectionsList = new ConnectionsList(new File(clientDir,
					"client.ini"));
			connectionsList
					.setToolTipText("Lists successful connections you have chosen to save. Once connected, click Store to save the connection to this list.");
			connectionsList
					.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			try {
				connectionsList.load();
			} catch (Exception e) {
				try {
					connectionsList.save();
				} catch (IOException e2) {
					JOptionPane.showMessageDialog(getFrame(),
							"Failed to load connection list.", "Warning",
							JOptionPane.WARNING_MESSAGE);
				}
			}
			connectionsList
					.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
						public void valueChanged(
								javax.swing.event.ListSelectionEvent e) {
							if (!e.getValueIsAdjusting()) {
								String s = connectionsList.getSelectedAddress();
								if (s != null && !s.equals("")) {
									getAddressTextField().setText(s);
									getRemoveConnectionButton()
											.setEnabled(true);
									getEditConnectionButton().setEnabled(true);
								} else {
									getRemoveConnectionButton().setEnabled(
											false);
									getEditConnectionButton().setEnabled(false);
								}
							}
						}
					});
			connectionsList.addKeyListener(new java.awt.event.KeyAdapter() {
				@Override
				public void keyPressed(java.awt.event.KeyEvent e) {
					if (e.getKeyCode() == KeyEvent.VK_ENTER) {
						getAddressTextField().requestFocusInWindow();
					}
				}
			});
		}
		return connectionsList;
	}

	/**
	 * This method initializes editConnectionButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getEditConnectionButton() {
		if (editConnectionButton == null) {
			editConnectionButton = new JButton();
			editConnectionButton.setText("Rename...");
			editConnectionButton
					.setToolTipText("Change the descriptive name of the selected connection.");
			editConnectionButton.setEnabled(false);
			editConnectionButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							String name = JOptionPane.showInputDialog(
									getFrame(),
									"Enter a new name for this connection.",
									getConnectionsList().getSelectedName());
							if (name != null && !name.equals("")) {
								getConnectionsList().setSelectedName(name);
								try {
									getConnectionsList().save();
								} catch (IOException f) {
									JOptionPane.showMessageDialog(getFrame(),
											"Failed to update list file.",
											"Warning",
											JOptionPane.WARNING_MESSAGE);
								}
							}
						}
					});
		}
		return editConnectionButton;
	}

	/**
	 * This method initializes removeConnectionButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getRemoveConnectionButton() {
		if (removeConnectionButton == null) {
			removeConnectionButton = new JButton();
			removeConnectionButton.setText("Remove");
			removeConnectionButton
					.setToolTipText("Permanently remove the connection from the list.");
			removeConnectionButton.setEnabled(false);
			removeConnectionButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(
								java.awt.event.ActionEvent evt) {
							int i = getConnectionsList().getSelectedIndex();
							if (i != -1) {
								int j = JOptionPane
										.showConfirmDialog(
												getFrame(),
												"This will permanently remove the selected connection. Are you sure?",
												"Remove Connection",
												JOptionPane.YES_NO_CANCEL_OPTION,
												JOptionPane.QUESTION_MESSAGE,
												Icons.CONNECT_DIALOG_ICON
														.getImageIcon());
								if (j == JOptionPane.YES_OPTION) {
									getConnectionsList().removeSelected();
									try {
										getConnectionsList().save();
									} catch (IOException e) {
										JOptionPane.showMessageDialog(
												getFrame(),
												"Failed to update list file.",
												"Warning",
												JOptionPane.WARNING_MESSAGE);
									}
								}
							}
						}
					});
		}
		return removeConnectionButton;
	}

	/**
	 * This method initializes newConnectionBox
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getNewConnectionBox() {
		if (newConnectionBox == null) {
			CardLayout cardLayout = new CardLayout();
			cardLayout.setHgap(0);
			cardLayout.setVgap(0);
			newConnectionBox = new JPanel();
			newConnectionBox.setBorder(BorderFactory
					.createTitledBorder("Make a Connection"));
			newConnectionBox.setLayout(cardLayout);
			newConnectionBox.add(getDisconnectedCard(), getDisconnectedCard()
					.getName());
			newConnectionBox.add(getConnectedCard(), getConnectedCard()
					.getName());
		}
		return newConnectionBox;
	}

	/**
	 * This method initializes disconnectedCard
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getDisconnectedCard() {
		if (disconnectedCard == null) {
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.fill = GridBagConstraints.BOTH;
			gridBagConstraints7.gridwidth = 3;
			gridBagConstraints7.gridx = 0;
			gridBagConstraints7.gridy = 0;
			gridBagConstraints7.weightx = 1.0;
			gridBagConstraints7.weighty = 0.0;
			gridBagConstraints7.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints6.gridy = 1;
			gridBagConstraints6.anchor = GridBagConstraints.EAST;
			gridBagConstraints6.gridx = 2;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints5.gridx = 1;
			gridBagConstraints5.gridy = 1;
			gridBagConstraints5.weightx = 1.0;
			gridBagConstraints5.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints4.gridy = 1;
			gridBagConstraints4.fill = GridBagConstraints.NONE;
			gridBagConstraints4.gridx = 0;
			addressLabel = new JLabel();
			addressLabel.setText("Address:");
			disconnectedCard = new JPanel();
			disconnectedCard.setLayout(new GridBagLayout());
			disconnectedCard.setName("disconnectedPanel");
			disconnectedCard.add(addressLabel, gridBagConstraints4);
			disconnectedCard.add(getAddressTextField(), gridBagConstraints5);
			disconnectedCard.add(getConnectButton(), gridBagConstraints6);
			disconnectedCard.add(getMakeConnectionDescriptionText(),
					gridBagConstraints7);
		}
		return disconnectedCard;
	}

	/**
	 * This method initializes makeConnectionDescriptionText
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getMakeConnectionDescriptionText() {
		if (makeConnectionDescriptionText == null) {
			makeConnectionDescriptionText = new JTextPane();
			makeConnectionDescriptionText.setOpaque(false);
			makeConnectionDescriptionText.setEditable(false);
			makeConnectionDescriptionText.setEnabled(false);
			makeConnectionDescriptionText
					.setDisabledTextColor(makeConnectionDescriptionText
							.getForeground());
			makeConnectionDescriptionText
					.setText("Enter the address of the distribution server and click Connect.");
		}
		return makeConnectionDescriptionText;
	}

	/**
	 * This method initializes addressTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getAddressTextField() {
		if (addressTextField == null) {
			addressTextField = new JTextField();
			addressTextField
					.setToolTipText("Enter here the address of the distributor to which you wish to connect.");
			addressTextField
					.addCaretListener(new javax.swing.event.CaretListener() {
						public void caretUpdate(javax.swing.event.CaretEvent e) {
							getConnectButton().setEnabled(
									!addressTextField.getText().equals(""));
						}
					});
			addressTextField
					.addFocusListener(new java.awt.event.FocusAdapter() {
						@Override
						public void focusGained(java.awt.event.FocusEvent e) {
							frame.getRootPane().setDefaultButton(
									getConnectButton());
						}

						@Override
						public void focusLost(FocusEvent e) {
							frame.getRootPane().setDefaultButton(null);
						}
					});
		}
		return addressTextField;
	}

	/**
	 * This method initializes connectButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getConnectButton() {
		if (connectButton == null) {
			connectButton = new JButton();
			connectButton.setText("Connect...");
			connectButton
					.setToolTipText("Connect to the specified address by opening the progress and authentication window.");
			connectButton.setEnabled(false);
			connectButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							worker.connect();
						}
					});
		}
		return connectButton;
	}

	/**
	 * This method initializes connectedCard
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getConnectedCard() {
		if (connectedCard == null) {
			GridBagConstraints gridBagConstraints25 = new GridBagConstraints();
			gridBagConstraints25.gridx = 2;
			gridBagConstraints25.anchor = GridBagConstraints.EAST;
			gridBagConstraints25.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints25.gridy = 0;
			GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
			gridBagConstraints14.anchor = GridBagConstraints.WEST;
			gridBagConstraints14.gridx = 1;
			gridBagConstraints14.gridy = 1;
			gridBagConstraints14.weightx = 1.0;
			gridBagConstraints14.insets = new Insets(4, 4, 4, 4);
			userValueLabel = new JLabel();
			userValueLabel.setText("username");
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			gridBagConstraints13.anchor = GridBagConstraints.WEST;
			gridBagConstraints13.gridx = 0;
			gridBagConstraints13.gridy = 1;
			gridBagConstraints13.insets = new Insets(4, 4, 4, 4);
			userLabel = new JLabel();
			userLabel.setText("User:");
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.anchor = GridBagConstraints.EAST;
			gridBagConstraints10.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints10.gridheight = 1;
			gridBagConstraints10.gridx = 2;
			gridBagConstraints10.gridy = 1;
			gridBagConstraints10.fill = GridBagConstraints.NONE;
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.anchor = GridBagConstraints.WEST;
			gridBagConstraints9.gridwidth = 2;
			gridBagConstraints9.gridx = 0;
			gridBagConstraints9.gridy = 0;
			gridBagConstraints9.weightx = 0.0;
			gridBagConstraints9.insets = new Insets(4, 4, 4, 4);
			connectedLabel = new JLabel();
			connectedLabel.setText("Connected");
			connectedCard = new JPanel();
			connectedCard.setLayout(new GridBagLayout());
			connectedCard.setName("connectedPanel");
			connectedCard.add(connectedLabel, gridBagConstraints9);
			connectedCard.add(getDisconnectButton(), gridBagConstraints10);
			connectedCard.add(userLabel, gridBagConstraints13);
			connectedCard.add(userValueLabel, gridBagConstraints14);
			connectedCard.add(getStoreConnectionButton(), gridBagConstraints25);
		}
		return connectedCard;
	}

	/**
	 * This method initializes storeConnectionButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getStoreConnectionButton() {
		if (storeConnectionButton == null) {
			storeConnectionButton = new JButton();
			storeConnectionButton.setText("Store");
			storeConnectionButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(
								java.awt.event.ActionEvent evt) {
							worker.storeConnection();
						}
					});
		}
		return storeConnectionButton;
	}

	/**
	 * This method initializes disconnectButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getDisconnectButton() {
		if (disconnectButton == null) {
			disconnectButton = new JButton();
			disconnectButton.setText("Disconnect");
			disconnectButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							worker.disconnect();
						}
					});
		}
		return disconnectButton;
	}

	/* -- Create / Edit Project Tab -- */

	/**
	 * This method initializes createEditProjectTab
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getCreateProjectTab() {
		if (createProjectTab == null) {
			GridBagConstraints gridBagConstraints33 = new GridBagConstraints();
			gridBagConstraints33.gridx = 0;
			gridBagConstraints33.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints33.anchor = GridBagConstraints.CENTER;
			gridBagConstraints33.gridwidth = 1;
			gridBagConstraints33.weightx = 1.0;
			gridBagConstraints33.weighty = 0.0;
			gridBagConstraints33.fill = GridBagConstraints.BOTH;
			gridBagConstraints33.gridy = 3;
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			gridBagConstraints12.gridx = 0;
			gridBagConstraints12.fill = GridBagConstraints.BOTH;
			gridBagConstraints12.weightx = 1.0;
			gridBagConstraints12.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints12.gridy = 0;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.fill = GridBagConstraints.BOTH;
			gridBagConstraints2.weightx = 1.0;
			gridBagConstraints2.weighty = 1.0;
			gridBagConstraints2.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints2.gridy = 1;
			createProjectTab = new JPanel();
			createProjectTab.setLayout(new GridBagLayout());
			createProjectTab.add(getFilesTasksBox(), gridBagConstraints2);
			createProjectTab.add(getProjectBox(), gridBagConstraints12);
			createProjectTab.add(getProjectSubmissionBox(),
					gridBagConstraints33);
		}
		return createProjectTab;
	}

	/**
	 * This method initializes projectSelectionBox
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getProjectBox() {
		if (projectBox == null) {
			GridBagConstraints gridBagConstraints37 = new GridBagConstraints();
			gridBagConstraints37.fill = GridBagConstraints.BOTH;
			gridBagConstraints37.gridy = 0;
			gridBagConstraints37.weightx = 1.0;
			gridBagConstraints37.weighty = 0.0;
			gridBagConstraints37.gridwidth = 2;
			gridBagConstraints37.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints37.gridx = 0;
			GridBagConstraints gridBagConstraints18 = new GridBagConstraints();
			gridBagConstraints18.anchor = GridBagConstraints.CENTER;
			gridBagConstraints18.gridx = 3;
			gridBagConstraints18.gridy = 1;
			gridBagConstraints18.weightx = 0.0;
			gridBagConstraints18.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints30 = new GridBagConstraints();
			gridBagConstraints30.gridx = 0;
			gridBagConstraints30.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints30.gridy = 1;
			projectLabel = new JLabel();
			projectLabel.setText("Project:");
			GridBagConstraints gridBagConstraints29 = new GridBagConstraints();
			gridBagConstraints29.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints29.gridy = 1;
			gridBagConstraints29.weightx = 1.0;
			gridBagConstraints29.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints29.gridx = 1;
			projectBox = new JPanel();
			projectBox.setLayout(new GridBagLayout());
			projectBox.setBorder(BorderFactory.createTitledBorder(null,
					"Project Name"));
			projectBox.add(getProjectNameField(), gridBagConstraints29);
			projectBox.add(projectLabel, gridBagConstraints30);
			projectBox.add(getProjectDescriptionText(), gridBagConstraints37);
		}
		return projectBox;
	}

	/**
	 * This method initializes selectionDescriptionText
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getProjectDescriptionText() {
		if (projectDescriptionText == null) {
			projectDescriptionText = new JTextPane();
			projectDescriptionText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			projectDescriptionText.setOpaque(false);
			projectDescriptionText.setText("Enter a name for your project.");
			projectDescriptionText
					.setDisabledTextColor(getProjectDescriptionText()
							.getForeground());
			projectDescriptionText.setEditable(false);
			projectDescriptionText.setEnabled(false);
		}
		return projectDescriptionText;
	}

	/**
	 * This method initializes projectsComboBox
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JTextField getProjectNameField() {
		if (projectNameField == null) {
			projectNameField = new JTextField("Project");
			projectNameField.addCaretListener(new CaretListener() {
				public void caretUpdate(CaretEvent e) {
					getProjectTreeModel().getRoot().setUserObject(
							projectNameField.getText());
				}
			});
		}
		return projectNameField;
	}

	/**
	 * This method initializes filesTasksBox
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getFilesTasksBox() {
		if (filesTasksBox == null) {
			GridBagConstraints gridBagConstraints35 = new GridBagConstraints();
			gridBagConstraints35.gridx = 1;
			gridBagConstraints35.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints35.gridwidth = 1;
			gridBagConstraints35.weightx = 1.0;
			gridBagConstraints35.fill = GridBagConstraints.NONE;
			gridBagConstraints35.anchor = GridBagConstraints.WEST;
			gridBagConstraints35.gridy = 3;
			localFilesLabel = new JLabel();
			localFilesLabel.setText("0 files to upload.");
			GridBagConstraints gridBagConstraints39 = new GridBagConstraints();
			gridBagConstraints39.gridx = 2;
			gridBagConstraints39.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints39.weightx = 0.5;
			gridBagConstraints39.anchor = GridBagConstraints.EAST;
			gridBagConstraints39.gridy = 3;
			GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
			gridBagConstraints31.gridx = 3;
			gridBagConstraints31.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints31.weightx = 0.0;
			gridBagConstraints31.gridy = 3;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.fill = GridBagConstraints.BOTH;
			gridBagConstraints3.gridy = 0;
			gridBagConstraints3.weightx = 1.0;
			gridBagConstraints3.weighty = 0.0;
			gridBagConstraints3.gridwidth = 4;
			gridBagConstraints3.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints3.gridheight = 1;
			gridBagConstraints3.gridx = 0;
			GridBagConstraints gridBagConstraints32 = new GridBagConstraints();
			gridBagConstraints32.fill = GridBagConstraints.BOTH;
			gridBagConstraints32.gridx = 2;
			gridBagConstraints32.gridy = 2;
			gridBagConstraints32.weightx = 0.5;
			gridBagConstraints32.weighty = 1.0;
			gridBagConstraints32.gridheight = 1;
			gridBagConstraints32.gridwidth = 2;
			gridBagConstraints32.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints20 = new GridBagConstraints();
			gridBagConstraints20.gridx = 0;
			gridBagConstraints20.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints20.anchor = GridBagConstraints.WEST;
			gridBagConstraints20.weightx = 1.0;
			gridBagConstraints20.fill = GridBagConstraints.NONE;
			gridBagConstraints20.gridy = 3;
			GridBagConstraints gridBagConstraints21 = new GridBagConstraints();
			gridBagConstraints21.fill = GridBagConstraints.BOTH;
			gridBagConstraints21.gridy = 2;
			gridBagConstraints21.weightx = 1.0;
			gridBagConstraints21.weighty = 1.0;
			gridBagConstraints21.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints21.gridheight = 1;
			gridBagConstraints21.gridwidth = 2;
			gridBagConstraints21.gridx = 0;
			filesTasksBox = new JPanel();
			filesTasksBox.setLayout(new GridBagLayout());
			filesTasksBox.setBorder(BorderFactory
					.createTitledBorder("Files and Tasks"));
			filesTasksBox
					.add(getProjectFilesScrollPane(), gridBagConstraints21);
			filesTasksBox.add(getAddFilesButton(), gridBagConstraints20);
			filesTasksBox.add(getTasksScrollPane(), gridBagConstraints32);
			filesTasksBox.add(getManagerDescriptionText(), gridBagConstraints3);
			filesTasksBox.add(getAddTaskButton(), gridBagConstraints31);
			filesTasksBox.add(getRemoveTaskButton(), gridBagConstraints39);
			filesTasksBox.add(localFilesLabel, gridBagConstraints35);
		}
		return filesTasksBox;
	}

	/**
	 * This method initializes managerDescriptionText
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getManagerDescriptionText() {
		if (managerDescriptionText == null) {
			managerDescriptionText = new JTextPane();
			managerDescriptionText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			managerDescriptionText.setOpaque(false);
			managerDescriptionText
					.setText("Organise the files that make up your project then select a source file (*.blend) and add tasks for rendering.");
			managerDescriptionText
					.setDisabledTextColor(getManagerDescriptionText()
							.getForeground());
			managerDescriptionText.setEditable(false);
			managerDescriptionText.setEnabled(false);
		}
		return managerDescriptionText;
	}

	/**
	 * This method initializes projectFilesScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getProjectFilesScrollPane() {
		if (projectFilesScrollPane == null) {
			projectFilesScrollPane = new JScrollPane();
			projectFilesScrollPane.setViewportView(getProjectFilesTree());
		}
		return projectFilesScrollPane;
	}

	/**
	 * This method initializes projectFilesTree
	 * 
	 * @return javax.swing.JTree
	 */
	private JTree getProjectFilesTree() {
		if (projectFilesTree == null) {
			DefaultTreeSelectionModel defaultTreeSelectionModel = new DefaultTreeSelectionModel();
			defaultTreeSelectionModel
					.setSelectionMode(DefaultTreeSelectionModel.SINGLE_TREE_SELECTION);
			projectFilesTree = new JTree();
			projectFilesTree.setShowsRootHandles(true);
			projectFilesTree.setModel(getProjectTreeModel());
			projectFilesTree.setCellRenderer(new IconedRenderers.TreeCell());
			projectFilesTree.setSelectionModel(defaultTreeSelectionModel);
		}
		return projectFilesTree;
	}

	/**
	 * This method initializes addFilesButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getAddFilesButton() {
		if (addFilesButton == null) {
			addFilesButton = new JButton();
			addFilesButton.setText("Add files...");
			addFilesButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							getBusyPane().setText("Add files...");
							getBusyPane().setVisible(true);
							try {
								FileManager.showFileManager(getFrame(),
										getProjectTreeModel().getRoot());
								getProjectTreeModel().reload();
								worker.recalculateFileCount();
								updateSubmitEnabled();
							} finally {
								getBusyPane().setVisible(false);
							}
						}
					});
		}
		return addFilesButton;
	}

	/**
	 * 
	 * @return ProjectTreeModel
	 */
	private ProjectTreeModel getProjectTreeModel() {
		if (projectFilesTreeModel == null) {
			projectFilesTreeModel = new ProjectTreeModel();
			projectFilesTreeModel.setRoot(new IconedNode(getProjectNameField()
					.getText(), Icons.PROJECT.getImageIcon()));
		}
		return projectFilesTreeModel;
	}

	/**
	 * This method initializes tasksScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getTasksScrollPane() {
		if (tasksScrollPane == null) {
			tasksScrollPane = new JScrollPane();
			tasksScrollPane.setViewportView(getTasksList());
		}
		return tasksScrollPane;
	}

	/**
	 * This method initializes tasksList
	 * 
	 * @return javax.swing.JList
	 */
	private JList getTasksList() {
		if (tasksList == null) {
			tasksList = new JList();
			tasksList.setModel(getTasksListModel());
			tasksList.setCellRenderer(new IconedRenderers.SingleIconListCell(
					Icons.TASK.getImageIcon()));
			tasksList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			tasksList
					.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
						public void valueChanged(
								javax.swing.event.ListSelectionEvent e) {
							if (tasksList.getSelectedIndex() == -1) {
								getRemoveTaskButton().setEnabled(false);
							} else {
								getRemoveTaskButton().setEnabled(true);
							}
						}
					});
		}
		return tasksList;
	}

	/**
	 * 
	 * @return DefaultListModel
	 */
	private DefaultListModel getTasksListModel() {
		if (tasksListModel == null) {
			tasksListModel = new DefaultListModel();
			tasksListModel
					.addListDataListener(new javax.swing.event.ListDataListener() {
						public void contentsChanged(
								javax.swing.event.ListDataEvent e) {
							updateSubmitEnabled();
						}

						public void intervalAdded(
								javax.swing.event.ListDataEvent e) {
							updateSubmitEnabled();
						}

						public void intervalRemoved(
								javax.swing.event.ListDataEvent e) {
							updateSubmitEnabled();
						}
					});
		}
		return tasksListModel;
	}

	/**
	 * This method initializes removeTaskButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getRemoveTaskButton() {
		if (removeTaskButton == null) {
			removeTaskButton = new JButton();
			removeTaskButton.setText("Remove task");
			removeTaskButton.setEnabled(false);
			removeTaskButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							int index = getTasksList().getSelectedIndex();
							if (index != -1) {
								getTasksListModel().remove(index);
								updateSubmitEnabled();
							}
						}
					});
		}
		return removeTaskButton;
	}

	/**
	 * This method initializes addTaskButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getAddTaskButton() {
		if (addTaskButton == null) {
			addTaskButton = new JButton();
			addTaskButton.setText("Add task...");
			addTaskButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								getBusyPane().setText("Add task...");
								getBusyPane().setVisible(true);
								if (!addTaskAction()) {
									JOptionPane.showMessageDialog(getFrame(),
											"Please select a source file "
													+ "first.", "Information",
											JOptionPane.INFORMATION_MESSAGE);
									getProjectFilesTree()
											.requestFocusInWindow();
								}
								updateSubmitEnabled();
							} finally {
								getBusyPane().setVisible(false);
							}
						}
					});
		}
		return addTaskButton;
	}

	private boolean addTaskAction() {

		/* Get selected source file. */
		TreePath path = getProjectFilesTree().getSelectionPath();
		if (path == null)
			return false;
		IconedNode node = (IconedNode) path.getLastPathComponent();
		Object o = node.getUserObject();
		if (!(o instanceof FileMap))
			return false;
		LocalPath local = ((FileMap) o).getDestinationPath();
		if (!local.getName().endsWith(".blend"))
			return false;
		Task task = AddTaskDialog.showAddTaskDialog(local);
		if (task != null) {
			getTasksListModel().addElement(task);
		}
		return true;
	}

	/**
	 * This method initializes projectSubmissionBox
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getProjectSubmissionBox() {
		if (projectSubmissionBox == null) {
			GridBagConstraints gridBagConstraints38 = new GridBagConstraints();
			gridBagConstraints38.fill = GridBagConstraints.BOTH;
			gridBagConstraints38.gridy = 0;
			gridBagConstraints38.weightx = 1.0;
			gridBagConstraints38.weighty = 0.0;
			gridBagConstraints38.gridwidth = 1;
			gridBagConstraints38.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints38.gridheight = 2;
			gridBagConstraints38.gridx = 0;
			GridBagConstraints gridBagConstraints34 = new GridBagConstraints();
			gridBagConstraints34.anchor = GridBagConstraints.CENTER;
			gridBagConstraints34.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints34.gridx = 1;
			gridBagConstraints34.gridy = 1;
			gridBagConstraints34.weightx = 0.0;
			projectSubmissionBox = new JPanel();
			projectSubmissionBox.setLayout(new GridBagLayout());
			projectSubmissionBox.setBorder(BorderFactory
					.createTitledBorder("Project Submission"));
			projectSubmissionBox.add(getSubmitButton(), gridBagConstraints34);
			projectSubmissionBox.add(getSubmissionDescriptionText(),
					gridBagConstraints38);
		}
		return projectSubmissionBox;
	}

	/**
	 * This method initializes submissionDescriptionText
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getSubmissionDescriptionText() {
		if (submissionDescriptionText == null) {
			submissionDescriptionText = new JTextPane();
			submissionDescriptionText.setCursor(new Cursor(
					Cursor.DEFAULT_CURSOR));
			submissionDescriptionText.setOpaque(false);
			submissionDescriptionText
					.setText("Send your project to the Distributor by clicking Submit. Once submitted the file and task lists will be cleared for further projects to be submitted.");
			submissionDescriptionText
					.setDisabledTextColor(getSubmissionDescriptionText()
							.getForeground());
			submissionDescriptionText.setEditable(false);
			submissionDescriptionText.setEnabled(false);
		}
		return submissionDescriptionText;
	}

	/**
	 * This method initializes submitButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getSubmitButton() {
		if (submitButton == null) {
			submitButton = new JButton();
			submitButton.setText("Submit...");
			submitButton.setEnabled(false);
			submitButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					worker.submit();
				}
			});
		}
		return submitButton;
	}

	/* -- Monitor projects tab. -- */

	/**
	 * This method initializes monitorProjectsTab
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getMonitorProjectsTab() {
		if (monitorProjectsTab == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.fill = GridBagConstraints.BOTH;
			gridBagConstraints11.weightx = 1.0;
			gridBagConstraints11.weighty = 1.0;
			gridBagConstraints11.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints11.gridy = 0;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 2;
			gridBagConstraints1.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.gridy = 0;
			monitorProjectsTab = new JPanel();
			monitorProjectsTab.setLayout(new GridBagLayout());
			monitorProjectsTab.add(getMonitorProjectsBox(),
					gridBagConstraints11);
		}
		return monitorProjectsTab;
	}

	/**
	 * This method initializes monitorProjectsBox
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getMonitorProjectsBox() {
		if (monitorProjectsBox == null) {
			GridBagConstraints gridBagConstraints40 = new GridBagConstraints();
			gridBagConstraints40.gridx = 1;
			gridBagConstraints40.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints40.gridy = 2;
			trafficIndicator = new JLabel(Icons.SIGNAL_OFF.getImageIcon());
			GridBagConstraints gridBagConstraints19 = new GridBagConstraints();
			gridBagConstraints19.gridx = 2;
			gridBagConstraints19.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints19.anchor = GridBagConstraints.EAST;
			gridBagConstraints19.gridy = 2;
			GridBagConstraints gridBagConstraints26 = new GridBagConstraints();
			gridBagConstraints26.fill = GridBagConstraints.BOTH;
			gridBagConstraints26.gridy = 0;
			gridBagConstraints26.weightx = 1.0;
			gridBagConstraints26.weighty = 0.0;
			gridBagConstraints26.gridwidth = 2;
			gridBagConstraints26.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints26.gridx = 1;
			GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
			gridBagConstraints17.fill = GridBagConstraints.BOTH;
			gridBagConstraints17.gridy = 1;
			gridBagConstraints17.weightx = 1.0;
			gridBagConstraints17.weighty = 1.0;
			gridBagConstraints17.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints17.gridwidth = 2;
			gridBagConstraints17.gridx = 1;
			monitorProjectsBox = new JPanel();
			monitorProjectsBox.setLayout(new GridBagLayout());
			monitorProjectsBox.setBorder(BorderFactory
					.createTitledBorder("Monitor Projects"));
			monitorProjectsBox.add(getMonitorListScrollPane(),
					gridBagConstraints17);
			monitorProjectsBox.add(getMonitoringDescriptionText(),
					gridBagConstraints26);
			monitorProjectsBox.add(getDeleteProjectButton(),
					gridBagConstraints19);
			monitorProjectsBox.add(trafficIndicator, gridBagConstraints40);
		}
		return monitorProjectsBox;
	}

	/**
	 * This method initializes monitoringDescriptionText
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getMonitoringDescriptionText() {
		if (monitoringDescriptionText == null) {
			monitoringDescriptionText = new JTextPane();
			monitoringDescriptionText.setOpaque(false);
			monitoringDescriptionText.setEditable(false);
			monitoringDescriptionText.setEnabled(false);
			monitoringDescriptionText
					.setDisabledTextColor(monitoringDescriptionText
							.getForeground());
			monitoringDescriptionText
					.setText("This panel is for monitoring your projects and for downloading the rendered frames.");
		}
		return monitoringDescriptionText;
	}

	/**
	 * This method initializes monitorListScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getMonitorListScrollPane() {
		if (monitorListScrollPane == null) {
			monitorListScrollPane = new JScrollPane();
			monitorListScrollPane.setViewportView(getMonitorList());
		}
		return monitorListScrollPane;
	}

	/**
	 * This method initializes monitorListTable
	 * 
	 * @return javax.swing.JTable
	 */
	private MonitorList getMonitorList() {
		if (monitorList == null) {
			monitorList = new MonitorList();
			monitorList.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					MonitorListItem mli = (MonitorListItem) ((JButton) e
							.getSource()).getParent();
					String projectName = mli.getProjectName();
					String command = e.getActionCommand();

					if (projectName.equals(""))
						return;

					if (command.equals(MonitorListItem.DOWNLOAD_ACTION_COMMAND)) {
						worker.downloadImages(projectName);
					} else if (command
							.equals(MonitorListItem.DELETE_ACTION_COMMAND)) {
						worker.deleteProject(projectName);
					}
				}
			});
		}
		return monitorList;
	}

	/**
	 * This method initializes deleteProjectButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getDeleteProjectButton() {
		if (deleteProjectButton == null) {
			deleteProjectButton = new JButton();
			deleteProjectButton.setText("Delete");
			deleteProjectButton.setEnabled(false);
		}
		return deleteProjectButton;
	}

	/**
	 * This method initializes archiveFileChooser
	 * 
	 * @return javax.swing.JFileChooser
	 */
	private JFileChooser getArchiveFileChooser() {
		if (archiveFileChooser == null) {
			archiveFileChooser = new JFileChooser();
			archiveFileChooser
					.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		}

		return archiveFileChooser;
	}

	/**
	 * 
	 * 
	 */
	private void updateSubmitEnabled() {
		boolean enabled = true;

		if (!SafeString.isSafe(getProjectNameField().getText()))
			enabled = false;

		if (getTasksListModel().getSize() == 0)
			enabled = false;

		if (!worker.isConnected())
			enabled = false;

		submitButton.setEnabled(enabled);
	}

	/**
	 * 
	 * @author Ian Thompson
	 * 
	 */
	private class Worker implements ConnectionListener {

		/**
		 * 
		 */
		public Worker() {
		}

		/**
		 * 
		 */
		private AuthorizedConnector connector = null;

		/**
		 * 
		 */
		public void connect() {
			getBusyPane().setText("Connecting...");
			getBusyPane().setVisible(true);
			try {
				connector = ConnectDialog.showConnectDialog(getFrame(),
						getAddressTextField().getText());
				connector.addConnectionListener(this);

				synchronized (connector) {
					if (isConnected() && connector.isAuthorized()) {
						userValueLabel.setText(connector.getUser());
						((CardLayout) getNewConnectionBox().getLayout()).show(
								getNewConnectionBox(), getConnectedCard()
										.getName());
					}
				}
			} finally {
				getBusyPane().setVisible(false);
			}
		}

		/**
		 * 
		 */
		public void disconnect() {
			if (connector != null) {
				connector.close();
				((CardLayout) getNewConnectionBox().getLayout()).show(
						getNewConnectionBox(), getDisconnectedCard().getName());
			}
		}

		/**
		 * @return the connector
		 */
		public AuthorizedConnector getConnector() {
			return connector;
		}

		/**
		 * 
		 * @return isConnected
		 */
		public boolean isConnected() {
			return connector != null && connector.isAuthorized();
		}

		/**
		 * 
		 */
		public void storeConnection() {
			if (connector != null) {
				String name = JOptionPane.showInputDialog(getFrame(),
						"Enter your chosen name for the connection to "
								+ connector.getHost() + ".", connector
								.getHost());
				if (name != null && !name.equals("")) {
					getConnectionsList().addConnection(name,
							connector.getHost(), connector.getPort());
					try {
						getConnectionsList().save();
					} catch (IOException e) {
						JOptionPane.showMessageDialog(getFrame(),
								"Failed to update list file.", "Warning",
								JOptionPane.WARNING_MESSAGE);
					}
				}
			}
		}

		/**
		 * 
		 * 
		 */
		public void submit() {
			getBusyPane().setText("Submitting new files and tasks...");
			getBusyPane().setVisible(true);
			String name = getProjectNameField().getText();
			Vector<FileMap> files = new Vector<FileMap>();
			IconedNode node, root = getProjectTreeModel().getRoot();
			Enumeration e = root.breadthFirstEnumeration();
			while (e.hasMoreElements()) {
				node = (IconedNode) e.nextElement();
				Object o = node.getUserObject();
				if (o instanceof FileMap) {
					/* Only add files, not directories. */
					if (!((FileMap) o).isDirectory())
						files.add((FileMap) o);
				}
			}
			FileMap[] filesArray = new FileMap[files.size()];
			files.toArray(filesArray);
			int i = 0;
			Task[] tasks = new Task[getTasksListModel().getSize()];
			Enumeration te = getTasksListModel().elements();
			while (te.hasMoreElements())
				tasks[i++] = (Task) te.nextElement();

			synchronized (connector) {
				boolean b = SubmitProjectDialog.submitProject(connector, name,
						filesArray, tasks);
				if (b) {
					getTasksListModel().clear();
					getProjectTreeModel().clear();
					recalculateFileCount();
				}
			}
			getBusyPane().setVisible(false);
		}

		/**
		 * 
		 */
		public void recalculateFileCount() {
			int count = 0;
			IconedNode node, root = getProjectTreeModel().getRoot();
			Enumeration<IconedNode> e = root.breadthFirstEnumeration();
			while (e.hasMoreElements()) {
				node = e.nextElement();
				Object o = node.getUserObject();
				if (o instanceof FileMap) {
					FileMap file = (FileMap) o;
					if (!file.getDestinationPath().isDirectory())
						count++;
				}
			}
			localFilesLabel.setText(count + " files to upload.");
		}

		/**
		 * 
		 * @param projectName
		 */
		public void downloadImages(String projectName) {
			getBusyPane().setText("Downloading images...");
			getBusyPane().setVisible(true);
			if (projectName != null && !projectName.equals("")) {
				int i = getArchiveFileChooser().showSaveDialog(getFrame());
				if (i == JFileChooser.APPROVE_OPTION) {
					String path = getArchiveFileChooser().getSelectedFile()
							.getAbsolutePath();
					File saveDir = new File(path);
					saveDir.mkdirs();
					synchronized (connector) {
						DownloadFramesDialog.downloadFrames(connector,
								projectName, saveDir);
					}
				}
			}
			getBusyPane().setVisible(false);
		}

		/**
		 * 
		 * @param projectName
		 */
		public void deleteProject(final String projectName) {
			if (projectName == null || projectName.equals(""))
				return;
			getBusyPane().setText("Deleting project...");
			getBusyPane().setVisible(true);
			new Thread(new Runnable() {
				public void run() {
					boolean success = false;
					synchronized (connector) {
						try {
							connector.send(new Submission.DeleteRequest(
									projectName));
							DeleteResponse response = (DeleteResponse) connector
									.receive();
							success = response.getSuccess();
						} catch (Exception e) {
						}
						if (success)
							JOptionPane.showMessageDialog(getFrame(),
									"Project deleted.", "Information",
									JOptionPane.INFORMATION_MESSAGE);
						else
							JOptionPane.showMessageDialog(getFrame(),
									"Cannot delete at this time.", "Warning",
									JOptionPane.WARNING_MESSAGE);
					}
					getBusyPane().setVisible(false);
				}
			}).start();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see net.sf.distriblend.client.event.ConnectionListener#connectionEstablished(net.sf.distriblend.client.event.ConnectionEvent)
		 */
		public void connectionEstablished(ConnectionEvent evt) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see net.sf.distriblend.client.event.ConnectionListener#connectionLost(net.sf.distriblend.client.event.ConnectionEvent)
		 */
		public void connectionLost(ConnectionEvent evt) {
			((CardLayout) getNewConnectionBox().getLayout()).show(
					getNewConnectionBox(), getDisconnectedCard().getName());
			JOptionPane.showMessageDialog(getFrame(),
					"Connection to Distributor has been disconnected.",
					"Information", JOptionPane.INFORMATION_MESSAGE);
			getTabbedPane().setSelectedIndex(0);
		}
	}

	private class Monitoring implements Runnable {

		private Thread thread = null;

		private boolean disabled = true;

		/**
		 * 
		 */
		public void enable() {
			if (thread == null) {
				thread = new Thread(this, "ClientProjectMonitoring");
				thread.setDaemon(true);
				disabled = false;
				thread.start();
			}
		}

		/**
		 * 
		 */
		public void disable() {
			if (thread != null) {
				disabled = true;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			try {
				while (!disabled) {
					if (worker.isConnected()) {
						AuthorizedConnector connector = worker.getConnector();

						/* Load from distributor. */
						MonitorItem[] monitorItems;
						synchronized (connector) {
							connector.send(new MonitorTasksRequest());
							trafficIndicator.setIcon(Icons.SIGNAL_LOW
									.getImageIcon());
							MonitorTasksResponse response = (MonitorTasksResponse) connector
									.receive();
							int count = response.getCount();
							monitorItems = new MonitorItem[count];
							for (int i = 0; i < count; i++)
								monitorItems[i] = (MonitorItem) connector
										.receive();
						}

						/* Update the table. */
						getMonitorList().updateWithList(monitorItems);
						trafficIndicator.setIcon(Icons.SIGNAL_HIGH
								.getImageIcon());
						Thread.sleep(500);
						trafficIndicator.setIcon(Icons.SIGNAL_OFF
								.getImageIcon());
						Thread.sleep(3000);

					} else {
						getMonitorList().clear();
						trafficIndicator.setIcon(Icons.SIGNAL_OFF
								.getImageIcon());
						disable();
					}
				}
			} catch (IOException e) {
				trafficIndicator.setIcon(Icons.SIGNAL_ERROR.getImageIcon());
				// TODO: Exception
			} catch (InterruptedException e) {
				// TODO: Exception
			} finally {
				trafficIndicator.setIcon(Icons.SIGNAL_OFF.getImageIcon());
				thread = null;
			}
		}
	}
}
