/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.security.NoSuchAlgorithmException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.WindowConstants;

import net.sf.distriblend.share.HashFunction;
import net.sf.distriblend.share.Settings;
import net.sf.distriblend.share.net.AuthorizedConnector;
import net.sf.distriblend.share.net.event.ConnectionEvent;
import net.sf.distriblend.share.net.event.ConnectionListener;

/**
 * @author Ian Thompson
 * 
 */
public class ConnectDialog extends JDialog {

	private JPanel dialogContentPane = null;

	private JPanel connectingPanel = null;

	private JTextPane statusInfoPane = null;

	private JProgressBar progressBar = null;

	private JPanel authorizationPanel = null;

	private JLabel usernameLabel = null;

	private JLabel passwordLabel = null;

	private JTextField userTextField = null;

	private JPasswordField passwordField = null;

	private JButton okButton = null;

	private JButton cancelAuthButton = null;

	private JTextPane authInfoPane = null;

	private JButton cancelButton = null;

	private final Worker worker;

	/**
	 * 
	 * @param owner
	 * @param host
	 * @return AuthorizedConnector
	 */
	public static AuthorizedConnector showConnectDialog(JFrame owner,
			String host) {
		int port = Settings.instance().getIntProperty(
				"connect.distributor.port");
		int timeout = Settings.instance().getIntProperty(
				"connect.distributor.timeout");
		AuthorizedConnector connector = new AuthorizedConnector(host, port,
				timeout);
		new ConnectDialog(owner, connector).setVisible(true);
		return connector;
	}

	/**
	 * This method initializes
	 * 
	 */
	private ConnectDialog(JFrame owner, AuthorizedConnector connector) {
		super(owner);
		initialize();
		this.worker = new Worker(connector);
		this.worker.start();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		this.setTitle("Connect");
		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.setModal(true);
		this.setSize(new Dimension(261, 149));
		this.setIconImage(Icons.CONNECT_DIALOG_ICON.getImage());
		this.setLocationRelativeTo(getOwner());
		this.setContentPane(getDialogContentPane());
	}

	/**
	 * This method initializes dialogContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getDialogContentPane() {
		if (dialogContentPane == null) {
			dialogContentPane = new JPanel();
			dialogContentPane.setLayout(new CardLayout());
			dialogContentPane.add(getConnectingPanel(), getConnectingPanel()
					.getName());
			dialogContentPane.add(getAuthorizationPanel(),
					getAuthorizationPanel().getName());
		}
		return dialogContentPane;
	}

	/**
	 * This method initializes connectingPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getConnectingPanel() {
		if (connectingPanel == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints11.weighty = 0.0;
			gridBagConstraints11.gridy = 2;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints1.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.weighty = 1.0;
			gridBagConstraints1.anchor = GridBagConstraints.NORTH;
			gridBagConstraints1.gridy = 1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.weightx = 1.0;
			gridBagConstraints.weighty = 1.0;
			gridBagConstraints.anchor = GridBagConstraints.SOUTH;
			gridBagConstraints.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints.gridx = 0;
			connectingPanel = new JPanel();
			connectingPanel.setLayout(new GridBagLayout());
			connectingPanel.setName("connectingPanel");
			connectingPanel.add(getStatusInfoPane(), gridBagConstraints);
			connectingPanel.add(getProgressBar(), gridBagConstraints1);
			connectingPanel.add(getCancelButton(), gridBagConstraints11);
		}
		return connectingPanel;
	}

	/**
	 * This method initializes statusInfoPane
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getStatusInfoPane() {
		if (statusInfoPane == null) {
			statusInfoPane = new JTextPane();
			statusInfoPane.setOpaque(false);
			statusInfoPane.setEditable(false);
			statusInfoPane.setEnabled(false);
			statusInfoPane.setDisabledTextColor(statusInfoPane.getForeground());
			statusInfoPane.setText("Connecting...");
		}
		return statusInfoPane;
	}

	/**
	 * This method initializes progressBar
	 * 
	 * @return javax.swing.JProgressBar
	 */
	private JProgressBar getProgressBar() {
		if (progressBar == null) {
			progressBar = new JProgressBar();
			progressBar.setValue(33);
			progressBar.setStringPainted(false);
		}
		return progressBar;
	}

	/**
	 * This method initializes authorizationPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getAuthorizationPanel() {
		if (authorizationPanel == null) {
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.fill = GridBagConstraints.BOTH;
			gridBagConstraints8.gridy = 0;
			gridBagConstraints8.weightx = 1.0;
			gridBagConstraints8.weighty = 0.0;
			gridBagConstraints8.gridwidth = 3;
			gridBagConstraints8.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints8.gridx = 0;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 2;
			gridBagConstraints7.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints7.anchor = GridBagConstraints.SOUTHEAST;
			gridBagConstraints7.weighty = 1.0;
			gridBagConstraints7.gridy = 3;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 1;
			gridBagConstraints6.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints6.weightx = 1.0;
			gridBagConstraints6.anchor = GridBagConstraints.SOUTHEAST;
			gridBagConstraints6.weighty = 1.0;
			gridBagConstraints6.gridy = 3;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.fill = GridBagConstraints.BOTH;
			gridBagConstraints5.gridy = 2;
			gridBagConstraints5.weightx = 1.0;
			gridBagConstraints5.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints5.gridwidth = 2;
			gridBagConstraints5.gridx = 1;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.fill = GridBagConstraints.BOTH;
			gridBagConstraints4.gridy = 1;
			gridBagConstraints4.weightx = 1.0;
			gridBagConstraints4.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints4.gridwidth = 2;
			gridBagConstraints4.gridx = 1;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints3.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints3.gridy = 2;
			passwordLabel = new JLabel();
			passwordLabel.setText("Password:");
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints2.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints2.gridy = 1;
			usernameLabel = new JLabel();
			usernameLabel.setText("User:");
			authorizationPanel = new JPanel();
			authorizationPanel.setLayout(new GridBagLayout());
			authorizationPanel.setName("authorizationPanel");
			authorizationPanel.add(usernameLabel, gridBagConstraints2);
			authorizationPanel.add(passwordLabel, gridBagConstraints3);
			authorizationPanel.add(getUserTextField(), gridBagConstraints4);
			authorizationPanel.add(getPasswordField(), gridBagConstraints5);
			authorizationPanel.add(getOkButton(), gridBagConstraints6);
			authorizationPanel.add(getCancelAuthButton(), gridBagConstraints7);
			authorizationPanel.add(getAuthInfoPane(), gridBagConstraints8);
		}
		return authorizationPanel;
	}

	/**
	 * This method initializes userTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getUserTextField() {
		if (userTextField == null) {
			userTextField = new JTextField();
			userTextField
					.addCaretListener(new javax.swing.event.CaretListener() {
						public void caretUpdate(javax.swing.event.CaretEvent e) {
							okButton.setEnabled(!userTextField.getText()
									.equals(""));
						}
					});
		}
		return userTextField;
	}

	/**
	 * This method initializes passwordField
	 * 
	 * @return javax.swing.JPasswordField
	 */
	private JPasswordField getPasswordField() {
		if (passwordField == null) {
			passwordField = new JPasswordField();
		}
		return passwordField;
	}

	/**
	 * This method initializes okButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText("OK");
			okButton.setEnabled(false);
			okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					worker.authorize();
				}
			});
		}
		return okButton;
	}

	/**
	 * This method initializes cancelAuthButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelAuthButton() {
		if (cancelAuthButton == null) {
			cancelAuthButton = new JButton();
			cancelAuthButton.setText("Cancel");
			cancelAuthButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							worker.cancelClose();
						}
					});
		}
		return cancelAuthButton;
	}

	/**
	 * This method initializes authInfoPane
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getAuthInfoPane() {
		if (authInfoPane == null) {
			authInfoPane = new JTextPane();
			authInfoPane.setOpaque(false);
			authInfoPane.setEditable(false);
			authInfoPane.setEnabled(false);
			authInfoPane.setDisabledTextColor(authInfoPane.getForeground());
			authInfoPane.setText("Enter your username and password.");
		}
		return authInfoPane;
	}

	/**
	 * This method initializes cancelButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText("Cancel");
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					worker.cancelClose();
				}
			});
		}
		return cancelButton;
	}

	private void showConnected() {
		getStatusInfoPane().setText("Connected.");
		getProgressBar().setValue(66);
		((CardLayout) getDialogContentPane().getLayout()).show(
				getDialogContentPane(), getAuthorizationPanel().getName());
		getRootPane().setDefaultButton(getOkButton());
		getUserTextField().requestFocusInWindow();
	}

	private void showFailed(String message) {
		getProgressBar().setValue(0);
		getStatusInfoPane().setText(message);
		getCancelButton().setText("Close");
		getRootPane().setDefaultButton(getCancelButton());
		getCancelButton().requestFocusInWindow();
	}

	private void showAuthorizing() {
		((CardLayout) getDialogContentPane().getLayout()).show(
				getDialogContentPane(), getConnectingPanel().getName());
		getStatusInfoPane().setText("Authorizing...");
	}

	/**
	 * Performs all network and generally time consuming operations to ensure
	 * the usability of the GUI is not lost.
	 * 
	 * @author Ian Thompson
	 * 
	 */
	private class Worker extends Thread implements ConnectionListener {

		private static final int ACTION_NONE = 0;

		private static final int ACTION_CONNECT = 1;

		private static final int ACTION_AUTHORIZE = 2;

		private static final int ACTION_EXIT = 3;

		private volatile int action = ACTION_NONE;

		private final AuthorizedConnector connector;

		/**
		 * @param connector
		 * 
		 */
		public Worker(AuthorizedConnector connector) {
			super("ConnectDialog.Worker [" + ConnectDialog.this + "]");
			this.connector = connector;
			this.setDaemon(true);
			this.connector.addConnectionListener(this);
		}

		/**
		 * 
		 */
		public void authorize() {
			if (this.action != ACTION_NONE)
				throw new IllegalStateException(
						"Cannot authorize. Another action is taking place.");
			this.action = ACTION_AUTHORIZE;
		}

		/**
		 * 
		 */
		public void cancelClose() {
			this.connector.close();
			dispose();
			this.action = ACTION_EXIT;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Thread#run()
		 */
		@Override
		public void run() {
			this.action = ACTION_CONNECT;

			while (action != ACTION_EXIT) {
				switch (action) {
				case ACTION_CONNECT:
					connector.connect();
					this.action = ACTION_NONE;
					break;
				case ACTION_AUTHORIZE:
					showAuthorizing();
					String hashed;
					try {
						hashed = HashFunction.getDefault().hash(
								new String(getPasswordField().getPassword()));
					} catch (NoSuchAlgorithmException e) {
						JOptionPane.showMessageDialog(ConnectDialog.this,
								"Error loading password hash algorithm.",
								"Error", JOptionPane.ERROR_MESSAGE);
						showFailed("Error");
						this.action = ACTION_EXIT;
						break;
					}
					boolean ok = connector.authorize(getUserTextField()
							.getText(), hashed);
					if (ok) {
						dispose();
						this.action = ACTION_EXIT;
					} else {
						showFailed("User name or password incorrect.");
						this.action = ACTION_NONE;
					}
					break;
				}
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see net.sf.distriblend.client.event.ConnectionListener#connectionEstablished(net.sf.distriblend.client.event.ConnectionEvent)
		 */
		public void connectionEstablished(ConnectionEvent evt) {
			showConnected();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see net.sf.distriblend.client.event.ConnectionListener#connectionLost(net.sf.distriblend.client.event.ConnectionEvent)
		 */
		public void connectionLost(ConnectionEvent evt) {
			showFailed("Failed to connect.");
		}
	}

}
