/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;
import java.net.URL;

import javax.swing.ImageIcon;

/**
 * @author Ian Thompson
 * 
 */
public enum Icons {

	/** */
	CLIENT_ICON("computer.png"),
	/** */
	CONNECT_DIALOG_ICON("server_connect.png"),
	/** */
	NEW_PROJECT_DIALOG_ICON("brick_add.png"),
	/** */
	FILE_MANAGER_DIALOG_ICON("folder.png"),
	/** */
	ADD_TASK_DIALOG_ICON("cog_add.png"),
	/** */
	SUBMIT_PROJECT_DIALOG_ICON("brick_go.png"),

	/** */
	TAB_CONNECT("server_connect.png"),
	/** */
	TAB_EDIT("brick_edit.png"),
	/** */
	TAB_MONITOR("brick_go.png"),

	/** */
	COMPUTER("computer.png"),
	/** */
	DRIVE_HARDDISK("drive.png"),
	/** */
	DRIVE_OPTICAL("drive_cd.png"),
	/** */
	FOLDER("folder.png"),
	/** */
	FILE_GENERIC("page_white.png"),
	/** */
	FILE_SCRIPT("page_white_text.png"),
	/** */
	FILE_IMAGE("picture.png"),
	/** */
	FILE_AUDIO("sound.png"),
	/** */
	FILE_VIDEO("film.png"),
	/** */
	FILE_BLEND("page_white_blend.png"),

	/** */
	CONNECTION_DISCONNECTED("disconnect.png"),
	/** */
	CONNECTION_CONNECTED("connect.png"),
	/** */
	PROJECT("brick.png"),
	/** */
	PROJECT_RUN("brick_go.png"),
	/** */
	PROJECT_ERROR("brick_error.png"),
	/** */
	TASK("cog.png"),
	/** */
	IMAGE_FORMAT("pictures.png"),
	/** */
	SIGNAL_ERROR("transmit_error.png"),
	/** */
	SIGNAL_OFF("transmit_off.png"),
	/** */
	SIGNAL_LOW("transmit_low.png"),
	/** */
	SIGNAL_HIGH("transmit_high.png");

	/**
	 * @param file
	 * @return ImageIcon
	 */
	public static Icons fromFileType(File file) {
		if (file == null)
			return FOLDER;
		return fromFileType(file.getName(), file.isDirectory());
	}

	/**
	 * @param name
	 * @param isDirectory
	 * @return ImageIcon
	 */
	public static Icons fromFileType(String name, boolean isDirectory) {
		name = name.toLowerCase();

		if (isDirectory) {
			if (name.length() == 0 || name.indexOf(":") == 1
					|| name.indexOf('/') == 0)
				return DRIVE_HARDDISK;
			return FOLDER;
		}

		if (name.endsWith(".blend")) {
			return FILE_BLEND;
		} else if (name.endsWith(".py") || name.endsWith(".pyc")
				|| name.endsWith(".txt")) {
			return Icons.FILE_SCRIPT;
		} else if (name.endsWith(".jpg") || name.endsWith(".jpeg")
				|| name.endsWith(".png") || name.endsWith(".bmp")
				|| name.endsWith(".gif") || name.endsWith(".ppm")
				|| name.endsWith(".pgm") || name.endsWith(".pgmyuv")
				|| name.endsWith(".pbm") || name.endsWith(".pam")
				|| name.endsWith(".rgb") || name.endsWith(".hdr")
				|| name.endsWith(".tga") || name.endsWith(".tif")
				|| name.endsWith(".tiff") || name.endsWith(".exr")
				|| name.endsWith(".cin") || name.endsWith(".dpx")) {
			return Icons.FILE_IMAGE;
		} else if (name.endsWith(".wav") || name.endsWith(".ogg")) {
			return Icons.FILE_AUDIO;
		} else if (name.endsWith(".avi") || name.endsWith(".dvd")
				|| name.endsWith(".mov") || name.endsWith(".wmv")) {
			return Icons.FILE_VIDEO;
		}
		return FILE_GENERIC;
	}

	/**
	 * Loads an {@link ImageIcon} from disk or jar file.
	 * 
	 * @return The resulting icon or <code>null</code> if none is found.
	 */
	public ImageIcon getImageIcon() {
		if (imageIcon == null)
			imageIcon = getImage() != null ? new ImageIcon(image) : null;
		return imageIcon;
	}

	/**
	 * Loads an {@link Image} from disk or jar file.
	 * 
	 * @return The resulting icon or <code>null</code> if none is found.
	 */
	public Image getImage() {
		if (image == null) {
			URL url = Icons.class.getResource(filename);
			image = url != null ? Toolkit.getDefaultToolkit().getImage(url)
					: null;
		}
		return image;
	}

	/* -- Private -- */

	private final String filename;

	private Image image = null;

	private ImageIcon imageIcon = null;

	private Icons(String filename) {
		this.filename = filename;
	}
}
