/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007-2008: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import net.sf.distriblend.share.protocol.Submission.MonitorItem;

/**
 * @author Ian Thompson
 * 
 */
public class MonitorList extends JPanel implements ActionListener {

	private final Vector<MonitorListItem> listItems;

	private final Vector<ActionListener> listeners;

	/**
	 * 
	 */
	private final TreeMap<String, MonitorItem> dataMap;

	/**
	 * 
	 */
	public MonitorList() {
		this.listeners = new Vector<ActionListener>();
		this.listItems = new Vector<MonitorListItem>();
		this.dataMap = new TreeMap<String, MonitorItem>();
		initialize();
	}

	/**
	 * 
	 * @param l
	 */
	public void addActionListener(ActionListener l) {
		listeners.add(l);
	}

	/**
	 * 
	 * @param l
	 */
	public void removeActionListener(ActionListener l) {
		listeners.remove(l);
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		this.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = Short.MAX_VALUE;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		this.add(new JPanel(), c);
	}

	/**
	 * 
	 * 
	 */
	public synchronized void clear() {
		synchronized (dataMap) {
			dataMap.clear();
			updateListItems();
		}
	}

	/**
	 * 
	 * @param items
	 */
	public void updateWithList(MonitorItem[] items) {
		/* Update data and resize items list. */
		synchronized (dataMap) {
			dataMap.clear();
			for (MonitorItem item : items)
				dataMap.put(item.projectName, item);
		}

		updateListItems();
	}

	/**
	 * Updates the GUI to reflect the data stored in the data map.
	 */
	protected void updateListItems() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				synchronized (dataMap) {

					int oldSize = listItems.size();
					int newSize = dataMap.size();

					/* Remove items if shrinking. */
					for (int i = oldSize - 1; i >= newSize; i--) {
						MonitorListItem item = listItems.get(i);
						item.removeActionListener(MonitorList.this);
						remove(item);
						listItems.remove(i);
					}

					/* Add items if growing. */
					for (int i = oldSize; i < newSize; i++) {
						MonitorListItem item = new MonitorListItem();
						listItems.add(i, item);
						GridBagConstraints c = new GridBagConstraints();
						c.gridx = 0;
						c.gridy = i;
						c.weightx = 1;
						c.weighty = 0;
						c.fill = GridBagConstraints.HORIZONTAL;
						add(item, c);
						item.addActionListener(MonitorList.this);
					}

					/* Update each item. */
					int i = 0;
					for (MonitorItem item : dataMap.values()) {
						listItems.get(i++).setMonitorData(item);
					}
				}

				repaint();
			};
		});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		/* Forward the event. */
		for (ActionListener l : listeners) {
			l.actionPerformed(e);
		}
	}
}
