/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007-2008: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.WindowConstants;

import net.sf.distriblend.share.FileMap;
import net.sf.distriblend.share.LocalPath;
import net.sf.distriblend.share.Task;
import net.sf.distriblend.share.net.Connector;
import net.sf.distriblend.share.net.event.TransferEvent;
import net.sf.distriblend.share.net.event.TransferListener;
import net.sf.distriblend.share.protocol.FileTransfer;
import net.sf.distriblend.share.protocol.Submission.CreateProjectRequest;
import net.sf.distriblend.share.protocol.Submission.CreateProjectResponse;
import net.sf.distriblend.share.protocol.Submission.SubmitFilesRequest;
import net.sf.distriblend.share.protocol.Submission.SubmitTasksRequest;

/**
 * @author Ian Thompson
 * 
 */
public class SubmitProjectDialog {

	/**
	 * 
	 * @param connector
	 * @param name
	 * @param files
	 * @param tasks
	 * @return complete
	 */
	public static boolean submitProject(Connector connector, String name,
			FileMap[] files, Task[] tasks) {
		return new SubmitProjectDialog(connector, name, files, tasks).worker
				.isComplete();
	}

	private Worker worker = null;

	private JDialog dialog = null; // @jve:decl-index=0:visual-constraint="54,34"

	private JPanel contentPane = null;

	private JPanel mainPanel = null;

	private JScrollPane logScrollPane = null;

	private JTextArea logTextArea = null;

	private JProgressBar uploadProgressBar = null;

	private JPanel buttonsPanel = null;

	private JButton doneButton = null;

	private JButton cancelButton = null;

	private JTextPane submitInfoPane = null;

	/**
	 * 
	 * @param connector
	 * @param projectSeed
	 */
	private SubmitProjectDialog(Connector connector, String name,
			FileMap[] files, Task[] tasks) {
		this.worker = new Worker(connector, name, files, tasks);
		this.worker.start();
		getDialog().setVisible(true);
	}

	/**
	 * This method initializes submitInfoPane
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getSubmitInfoPane() {
		if (submitInfoPane == null) {
			submitInfoPane = new JTextPane();
			submitInfoPane.setEnabled(false);
			submitInfoPane.setDisabledTextColor(getSubmitInfoPane()
					.getForeground());
			submitInfoPane
					.setText("Your project is being sent to the Distributor. This may take some time. To cancel the transfer click Cancel. Any files already sent will not be deleted in case you wish to continue the transfer later.");
			submitInfoPane.setOpaque(false);
			submitInfoPane.setEditable(false);
		}
		return submitInfoPane;
	}

	/**
	 * This method initializes dialog
	 * 
	 * @return javax.swing.JDialog
	 */
	private JDialog getDialog() {
		if (dialog == null) {
			dialog = new JDialog();
			dialog.setModal(true);
			dialog.setSize(new Dimension(383, 299));
			dialog.setTitle("Submit Project");
			dialog
					.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			dialog.setContentPane(getContentPane());
			dialog.setIconImage(Icons.SUBMIT_PROJECT_DIALOG_ICON.getImage());
		}
		return dialog;
	}

	/**
	 * This method initializes contentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getContentPane() {
		if (contentPane == null) {
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints3.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints3.gridy = 1;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.ipadx = 0;
			gridBagConstraints1.ipady = 0;
			gridBagConstraints1.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.weighty = 1.0;
			gridBagConstraints1.fill = GridBagConstraints.BOTH;
			gridBagConstraints1.gridy = 0;
			contentPane = new JPanel();
			contentPane.setLayout(new GridBagLayout());
			contentPane.add(getMainPanel(), gridBagConstraints1);
			contentPane.add(getButtonsPanel(), gridBagConstraints3);
		}
		return contentPane;
	}

	/**
	 * This method initializes mainPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getMainPanel() {
		if (mainPanel == null) {
			GridBagConstraints gridBagConstraints41 = new GridBagConstraints();
			gridBagConstraints41.fill = GridBagConstraints.BOTH;
			gridBagConstraints41.gridy = 0;
			gridBagConstraints41.weightx = 1.0;
			gridBagConstraints41.weighty = 0.0;
			gridBagConstraints41.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints41.gridx = 0;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints2.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints2.gridy = 3;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.BOTH;
			gridBagConstraints.gridy = 1;
			gridBagConstraints.weightx = 1.0;
			gridBagConstraints.weighty = 1.0;
			gridBagConstraints.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints.gridx = 0;
			mainPanel = new JPanel();
			mainPanel.setLayout(new GridBagLayout());
			mainPanel.setBorder(BorderFactory
					.createTitledBorder("Project Transfer"));
			mainPanel.add(getLogScrollPane(), gridBagConstraints);
			mainPanel.add(getUploadProgressBar(), gridBagConstraints2);
			mainPanel.add(getSubmitInfoPane(), gridBagConstraints41);
		}
		return mainPanel;
	}

	/**
	 * This method initializes logScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getLogScrollPane() {
		if (logScrollPane == null) {
			logScrollPane = new JScrollPane();
			logScrollPane.setViewportView(getLogTextArea());
		}
		return logScrollPane;
	}

	/**
	 * This method initializes logTextArea
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextArea getLogTextArea() {
		if (logTextArea == null) {
			logTextArea = new JTextArea();
			logTextArea.setEditable(false);
			logTextArea.setFont(getSubmitInfoPane().getFont());
		}
		return logTextArea;
	}

	/**
	 * This method initializes uploadProgressBar
	 * 
	 * @return javax.swing.JProgressBar
	 */
	private JProgressBar getUploadProgressBar() {
		if (uploadProgressBar == null) {
			uploadProgressBar = new JProgressBar();
			uploadProgressBar.setStringPainted(true);
		}
		return uploadProgressBar;
	}

	/**
	 * This method initializes buttonsPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getButtonsPanel() {
		if (buttonsPanel == null) {
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 1;
			gridBagConstraints5.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints5.gridy = 0;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints4.weightx = 1.0;
			gridBagConstraints4.anchor = GridBagConstraints.EAST;
			buttonsPanel = new JPanel();
			buttonsPanel.setLayout(new GridBagLayout());
			buttonsPanel.add(getDoneButton(), gridBagConstraints4);
			buttonsPanel.add(getCancelButton(), gridBagConstraints5);
		}
		return buttonsPanel;
	}

	/**
	 * This method initializes doneButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getDoneButton() {
		if (doneButton == null) {
			doneButton = new JButton();
			doneButton.setText("Finish");
			doneButton.setEnabled(false);
			doneButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					getDialog().dispose();
				}
			});
		}
		return doneButton;
	}

	/**
	 * This method initializes cancelButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText("Cancel");
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					worker.cancel();
					getDialog().dispose();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * 
	 * @author Ian Thompson
	 * 
	 */
	private class Worker extends Thread {

		/**
		 * 
		 */
		private boolean complete = false;

		/**
		 * 
		 */
		private final Connector connector;

		private String name;
		private final FileMap[] files;
		private final Task[] tasks;

		/**
		 * 
		 * @param connector
		 * @param projectSeed
		 */
		public Worker(Connector connector, String name, FileMap[] files,
				Task[] tasks) {
			this.connector = connector;
			this.name = name;
			this.files = files;
			this.tasks = tasks;
		}

		/**
		 * @return the complete
		 */
		public boolean isComplete() {
			return complete;
		}

		/**
		 * 
		 * 
		 */
		public void cancel() {

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Thread#run()
		 */
		@Override
		public void run() {

			final JProgressBar pb = getUploadProgressBar();

			/* Initialize. */
			getLogTextArea().setText("Initializing...\n");
			pb.setValue(0);
			if (!connector.isSuccessfullyConnected()) {
				getDialog().dispose();
				appendToLog("Failed.");
				JOptionPane
						.showMessageDialog(
								getDialog(),
								"Connection broken. Try reconnecting before submitting your project.",
								"Connect", JOptionPane.WARNING_MESSAGE,
								Icons.CONNECTION_DISCONNECTED.getImageIcon());
				return;
			}
			final int count = files.length;
			pb.setValue(20);

			/* Create project. */
			try {
				do {
					connector.send(new CreateProjectRequest(name));
					pb.setValue(50);

					Object o = connector.receive();
					if (!(o instanceof CreateProjectResponse)) {
						appendToLog("Failed.\n");
						pb.setValue(0);
						JOptionPane.showMessageDialog(getDialog(),
								"Error in protocol. Try reconnecting.",
								"Warning", JOptionPane.WARNING_MESSAGE);
						return;
					}
					CreateProjectResponse response = (CreateProjectResponse) o;

					if (response.isSuccessful())
						break;

					name = JOptionPane
							.showInputDialog(
									getDialog(),
									"Project name \""
											+ name
											+ "\" is not unique. Please enter a new name:",
									name);
					if (name == null)
						break;
				} while (true);

				if (name == null) {
					appendToLog("Failed.\n");
					pb.setValue(0);
					JOptionPane.showMessageDialog(getDialog(),
							"Failed to create new project.", "Warning",
							JOptionPane.WARNING_MESSAGE);
					return;
				}

				pb.setValue(100);
				appendToLog("Done.\n");

			} catch (IOException e) {
				appendToLog("Failed.\n");
				pb.setValue(0);
				JOptionPane.showMessageDialog(getDialog(),
						"Error in communication. Try reconnecting.", "Warning",
						JOptionPane.WARNING_MESSAGE);
				return;
			}

			/* Submit files. */
			appendToLog("Uploading files...\n");
			pb.setValue(0);
			if (files.length > 0) {
				try {
					connector.send(new SubmitFilesRequest(count));

					/* Upload files. */
					pb.setValue(50); // TODO: Proper progress bar.

					ObjectOutputStream out = connector.getOutputStream();
					FileTransfer.Header header;
					LocalPath path;
					File file;
					BufferedInputStream in;
					for (int i = 0; i < count; i++) {
						path = files[i].getDestinationPath();
						file = files[i].getSourcePath();
						final long size = file.length();
						final int ii = i;
						in = new BufferedInputStream(new FileInputStream(file));
						header = new FileTransfer.Header(path, size);
						connector.send(header);

						TransferListener l = new TransferListener() {
							public void progressChanged(TransferEvent event) {
								long t = event.getTransferred();
								int value = 100;
								if (size > 0)
									value = (int) (100 * (t / size + ii) / count);
								pb.setValue(value);
							}
						};
						FileTransfer.transferData(in, out, size, l);
						in.close();
						pb.setValue(100 * i / count);
					}
					connector.receive();
					pb.setValue(100);
					appendToLog("Upload complete.\n");

				} catch (IOException e) {
					appendToLog("Failed.\n");
					pb.setValue(0);
					e.printStackTrace();
					JOptionPane.showMessageDialog(getDialog(),
							"Error in communication. Try reconnecting.",
							"Warning", JOptionPane.WARNING_MESSAGE);
					return;
				}
			} else {
				appendToLog("Done. (No files to send)\n");
				pb.setValue(100);
			}

			/* Send tasks. */
			appendToLog("Sending tasks...\n");
			pb.setValue(0);
			if (tasks.length > 0) {
				try {
					connector.send(new SubmitTasksRequest(tasks));
					pb.setValue(50);
					connector.receive();
					pb.setValue(100);
					appendToLog("Done.\n");

				} catch (IOException e) {
					appendToLog("Failed.\n");
					pb.setValue(0);
					JOptionPane.showMessageDialog(getDialog(),
							"Error in communication. Try reconnecting.",
							"Warning", JOptionPane.WARNING_MESSAGE);
					return;
				}
			} else {
				appendToLog("Done. (No tasks to send)\n");
				pb.setValue(100);
			}

			/* Complete. */
			complete = true;
			appendToLog("Finished.\n");
			pb.setValue(100);
			getCancelButton().setEnabled(false);
			getDoneButton().setEnabled(true);
		}

		/**
		 * 
		 * @param s
		 */
		private void appendToLog(final String s) {
			getLogTextArea().append(s);
			getLogScrollPane().getVerticalScrollBar().setValue(
					Integer.MAX_VALUE);
			getLogTextArea().repaint();
		}
	}
}
