/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.io.File;
import java.util.Enumeration;

import javax.swing.JTree;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.ExpandVetoException;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;


/**
 * @author Ian Thompson
 * 
 */
public class SystemFileTree extends JTree implements TreeWillExpandListener {

	private DefaultTreeModel model = null;

	private boolean directoriesOnly = false;

	private static final class UNPOPULATED extends IconedNode {
	};

	/**
	 * @param directoriesOnly
	 */
	public SystemFileTree(boolean directoriesOnly) {
		super();
		this.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		this.setShowsRootHandles(true);
		this.directoriesOnly = directoriesOnly;
		this.model = new DefaultTreeModel(null);
		this.setModel(model);
		this.setCellRenderer(new IconedRenderers.TreeCell());
		this.addTreeWillExpandListener(this);
	}

	/**
	 * 
	 */
	public SystemFileTree() {
		this(false);
	}

	/**
	 * @param path
	 */
	public void setRootPath(File path) {
		IconedNode root = new IconedNode(path, Icons.fromFileType(path)
				.getImageIcon());
		root.add(new UNPOPULATED());
		populate(root);
		model.setRoot(root);
		expandPath(new TreePath(root.getPath()));
	}

	/**
	 * @param file
	 */
	public void selectChild(File file) {
		IconedNode node = (IconedNode) getLastSelectedPathComponent();
		Enumeration<IconedNode> children = node.children();
		while (children.hasMoreElements()) {
			IconedNode child = children.nextElement();
			if (child.getUserObject().equals(file)) {
				setSelectionPath(new TreePath(child.getPath()));
				break;
			}
		}
	}

	public void treeWillCollapse(TreeExpansionEvent event)
			throws ExpandVetoException {
	}

	public void treeWillExpand(TreeExpansionEvent event)
			throws ExpandVetoException {
		IconedNode node = (IconedNode) event.getPath().getLastPathComponent();
		populate(node);
		this.scrollPathToVisible(new TreePath(node.getPath()));
	}

	/**
	 * @param node
	 */
	public void populate(IconedNode node) {
		if (node.getChildCount() == 1
				&& node.getChildAt(0) instanceof UNPOPULATED) {
			node.removeAllChildren();
			Object o = node.getUserObject();
			if (!(o instanceof File))
				return;
			File dir = (File) o;
			if (dir.isDirectory()) {
				File[] files = dir.listFiles();
				java.util.Arrays.sort(files);
				int count = 0;
				for (int i = 0; i < files.length; i++) {
					if (!directoriesOnly || files[i].isDirectory()) {
						File f = files[i];
						IconedNode child = new IconedNode(f, Icons
								.fromFileType(f).getImageIcon());
						if (f.isDirectory())
							child.add(new UNPOPULATED());
						node.insert(child, count);
						count++;
					}
				}
				int[] inserted = new int[count];
				for (int i = 0; i < count; i++)
					inserted[i] = i;
				model.nodesWereInserted(node, inserted);
			}
		}
	}
}
