/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007-2008: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.distributor.gui;

import java.util.Vector;

import javax.swing.AbstractListModel;

/**
 * Presents a sorted list model whereby new elements are added to the list only
 * if they do not occur already.
 * 
 * @author Ian Thompson
 * 
 */
public class SingleSortedListModel extends AbstractListModel {

	public SingleSortedListModel() {
		this.dataVector = new Vector<Object>();
	}

	public void addElement(Object e) {
		for (int i = 0; i < getSize(); i++) {
			Object item = dataVector.elementAt(i);
			int comparison = e.toString().compareTo(item.toString());
			if (comparison == 0)
				return;
			else if (comparison < 0) {
				dataVector.insertElementAt(e, i);
				fireIntervalAdded(this, i, i);
				return;
			}
		}
		dataVector.add(e);
		fireIntervalAdded(this, getSize(), getSize());
	}

	public void removeElementAt(int index) {
		dataVector.remove(index);
		fireIntervalRemoved(this, index, index);
	}

	public void clear() {
		int size = getSize();
		dataVector.clear();
		fireIntervalRemoved(this, 0, size);
	}

	public Object getElementAt(int index) {
		return dataVector.elementAt(index);
	}

	public int getSize() {
		return dataVector.size();
	}

	private final Vector<Object> dataVector;

}
