/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

/**
 * 
 * <p>
 * Presents a system of records that are mirrored by a file structure on disk.
 * The general idea of these classes is to persist data across instances of the
 * application. To achieve this, records are loaded from disk at start-up and as
 * more records are added they are serialized and saved. This 'read at startup,
 * write while running' paradigm together with synchronization ensures a safe
 * storage mechanism.
 * </p>
 * <p>
 * The storage system uses the following path structures (replacing
 * <code>/</code> with the system file separator
 * {@link java.io.File#separator}): <table border="1">
 * <tr>
 * <td><code>{ROOT}</code></td>
 * <td>Storage root directory. Taken from the <code>"storage.root"</code>
 * property of {@link net.sf.distriblend.share.Settings}. Holds sub-directories
 * that represent user names. This directory must allow the creation of
 * sub-directories if new users are to be added and should not contain any
 * non-user directories.</td>
 * </tr>
 * <tr>
 * <td><code>{ROOT}/{USER}</code></td>
 * <td>User's storage directory (where {USER} is replaced by the user's name).
 * This directory must hold a <code>"user.ini"</code> file and be writable by
 * the Distributor application.</td>
 * </tr>
 * <tr>
 * <td><code>{ROOT}/{USER}/user.ini</code></td>
 * <td>User profile. Holds user password and preference settings.</td>
 * </tr>
 * <tr>
 * <td><code>{ROOT}/{USER}/projects</code></td>
 * <td>Projects directory. Contains sub-directories for each project.</td>
 * </tr>
 * <tr>
 * <td><code>{ROOT}/{USER}/projects/{PROJECT}</code></td>
 * <td>Project storage directory. Holds the sub-directories as follows...</td>
 * </tr>
 * <tr>
 * <td><code>{ROOT}/{USER}/projects/{PROJECT}/archives</code></td>
 * <td>Project archives directory. Stores uploaded archive files in the form
 * <code>"archive_{NUM}.zip"</code> where {NUM} is a unique number to allow
 * for multiple file sets to be uploaded. The first file archive will always be
 * <code>"archive_0.zip"</code>. While uploading <code>"_upload"</code> is
 * appended to the file name. This is removed once the upload has completed
 * successfully.</td>
 * </tr>
 * <tr>
 * <td><code>{ROOT}/{USER}/projects/{PROJECT}/tasks</code></td>
 * <td>Project tasks directory. Stores and maintains a list of tasks that have
 * been submitted and their progresses. Each task is stored in the format
 * <code>"task_{NUM}.task"</code> where {NUM} is a unique number incremented
 * each time a task is added.</td>
 * </tr>
 * <tr>
 * <td><code>{ROOT}/{USER}/projects/{PROJECT}/frames</code></td>
 * <td>Frame storage directory. Holds sub-directories for each source file,
 * created when frames are first received.</td>
 * </tr>
 * <tr>
 * <td><code>{ROOT}/{USER}/projects/{PROJECT}/frames/{SOURCEFILE}</code></td>
 * <td>Rendered frames directory. Holds the actual frame files received from
 * the nodes network. These files are of the form <code>"{FRMNUM}.{EXT}"</code>,
 * where {FRMNUM} is the formatted frame number and {EXT} is the chosen
 * extension (for example, <code>"0001.png"</code>).</td>
 * </tr>
 * </table>
 * </p>
 */
package net.sf.distriblend.distributor.storage;

