/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007-2008: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.node.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;
import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.border.TitledBorder;

import net.sf.distriblend.node.Renderer;
import net.sf.distriblend.share.Settings;

class WizardStep2 extends WizardStep {

	private JTextPane blenderCommandText = null;
	private JTextField blenderCommandField = null;
	private JButton browseButton = null;
	private JTextArea logText = null;
	private JScrollPane logScrollPane = null;
	private JButton testButton = null;
	private JFileChooser fileChooser = null;

	private boolean testedPositive = false;

	/**
	 * This method initializes
	 * 
	 */
	public WizardStep2() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		GridBagConstraints gridBagConstraints40 = new GridBagConstraints();
		gridBagConstraints40.gridx = 0;
		gridBagConstraints40.anchor = GridBagConstraints.WEST;
		gridBagConstraints40.insets = new Insets(4, 4, 4, 4);
		gridBagConstraints40.gridwidth = 2;
		gridBagConstraints40.gridy = 2;
		GridBagConstraints gridBagConstraints39 = new GridBagConstraints();
		gridBagConstraints39.fill = GridBagConstraints.BOTH;
		gridBagConstraints39.gridy = 3;
		gridBagConstraints39.weightx = 1.0;
		gridBagConstraints39.weighty = 1.0;
		gridBagConstraints39.insets = new Insets(4, 4, 4, 4);
		gridBagConstraints39.gridwidth = 2;
		gridBagConstraints39.gridx = 0;
		GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
		gridBagConstraints14.gridx = 1;
		gridBagConstraints14.insets = new Insets(4, 4, 16, 16);
		gridBagConstraints14.fill = GridBagConstraints.VERTICAL;
		gridBagConstraints14.gridy = 1;
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.fill = GridBagConstraints.BOTH;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.insets = new Insets(4, 16, 16, 4);
		gridBagConstraints1.gridx = 0;
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.weighty = 0.0;
		gridBagConstraints.insets = new Insets(4, 4, 4, 4);
		gridBagConstraints.gridwidth = 2;
		gridBagConstraints.gridx = 0;
		this.setLayout(new GridBagLayout());
		this.setBorder(new TitledBorder(null, "Step 2 of 4",
				TitledBorder.TRAILING, TitledBorder.TOP));
		this.add(getBlenderCommandText(), gridBagConstraints);
		this.add(getBlenderCommandField(), gridBagConstraints1);
		this.add(getBrowseButton(), gridBagConstraints14);
		this.add(getLogScrollPane(), gridBagConstraints39);
		this.add(getTestButton(), gridBagConstraints40);
	}

	/**
	 * This method initializes blenderCommandText
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getBlenderCommandText() {
		if (blenderCommandText == null) {
			blenderCommandText = new JTextPane();
			blenderCommandText.setOpaque(false);
			blenderCommandText.setEditable(false);
			blenderCommandText.setText("Enter the command used to execute "
					+ "blender:");
		}
		return blenderCommandText;
	}

	/**
	 * This method initializes blenderCommandField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getBlenderCommandField() {
		if (blenderCommandField == null) {
			blenderCommandField = new JTextField();
			String s = "C:\\Program Files\\Blender Foundation\\Blender\\"
					+ "blender.exe";
			if (new File(s).exists())
				blenderCommandField.setText(s);
			else
				blenderCommandField.setText("blender");
		}
		return blenderCommandField;
	}

	/**
	 * This method initializes blenderCommandBrowseButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getBrowseButton() {
		if (browseButton == null) {
			browseButton = new JButton();
			browseButton.setText("Browse...");
			browseButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					String path = getBlenderCommandField().getText();
					getFileChooser().setSelectedFile(new File(path));
					int result = getFileChooser().showOpenDialog(
							WizardStep2.this);
					if (result == JFileChooser.APPROVE_OPTION) {
						path = getFileChooser().getSelectedFile()
								.getAbsolutePath();
						getBlenderCommandField().setText(path);
					}
				}
			});
		}
		return browseButton;
	}

	/**
	 * This method initializes directoryChooser
	 * 
	 * @return javax.swing.JFileChooser
	 */
	private JFileChooser getFileChooser() {
		if (fileChooser == null) {
			fileChooser = new JFileChooser();
			fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		}
		return fileChooser;
	}

	/**
	 * This method initializes logText
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextArea getLogText() {
		if (logText == null) {
			logText = new JTextArea();
			logText.setOpaque(false);
			logText.setEditable(false);

			Logger l = Logger.getLogger(Renderer.class.getName());
			l.addHandler(new Handler() {
				@Override
				public void close() throws SecurityException {
				}

				@Override
				public void flush() {
				}

				@Override
				public void publish(LogRecord record) {
					getLogText().append(record.getMessage() + "\n");
				}
			});
		}
		return logText;
	}

	/**
	 * This method initializes logScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getLogScrollPane() {
		if (logScrollPane == null) {
			logScrollPane = new JScrollPane();
			logScrollPane.setViewportView(getLogText());
			logScrollPane.setBorder(BorderFactory.createEtchedBorder());
		}
		return logScrollPane;
	}

	/**
	 * This method initializes testButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getTestButton() {
		if (testButton == null) {
			testButton = new JButton();
			testButton.setText("Test");
			testButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					/* Check blender can be launched. */
					File f = new File(getBlenderCommandField().getText());
					Settings.instance().setStringProperty("blender.command",
							f.getPath());
					logText.setText("Results\n-------------------------------"
							+ "---------\n");
					if (!Renderer.instance().test()) {
						testedPositive = false;
						String message;
						if (!f.exists())
							message = "Cannot find blender executable.";
						else
							message = "Blender executable failed to start.";
						JOptionPane.showMessageDialog(WizardStep2.this,
								message, "Test Configuration",
								JOptionPane.ERROR_MESSAGE);
					} else
						testedPositive = true;
				}
			});
		}
		return testButton;
	}

	private void updateSettings() {
		Settings set = Settings.instance();
		set.setStringProperty("blender.command", getBlenderCommandField()
				.getText());
	}

	@Override
	boolean validateForm() {
		if (!testedPositive) {
			int result = JOptionPane
					.showConfirmDialog(
							this,
							"Are you "
									+ "sure you wish to continue without testing the renderer?",
							"Render Test", JOptionPane.YES_NO_CANCEL_OPTION,
							JOptionPane.WARNING_MESSAGE);

			if (result != JOptionPane.YES_OPTION)
				return false;
		}

		updateSettings();
		return true;
	}
}
