/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007-2008: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.share;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * A function for producing secure hashes, suitable for hiding password Strings.
 * 
 * @author Ian Thompson
 * 
 */
public abstract class HashFunction {

	/**
	 * A default hash function.
	 */
	private static final HashFunction DEFAULT_HASH_FUNCTION = new HashFunction() {
		@Override
		public String hash(String s) throws NoSuchAlgorithmException {
			MessageDigest md = MessageDigest.getInstance("SHA");
			md.update(s.getBytes());
			byte[] raw = md.digest();
			return (Base64Encoder.encode(raw));
		}
	};

	/**
	 * Returns the default {@code HashFunction}.
	 * 
	 * @return the default {@code HashFunction}.
	 */
	public static HashFunction getDefault() {
		return DEFAULT_HASH_FUNCTION;
	}

	/**
	 * Hashes the specified {@link String}.
	 * 
	 * @param s
	 *            The String to hash.
	 * @return A hashed version of the String.
	 * @throws NoSuchAlgorithmException
	 */
	public abstract String hash(String s) throws NoSuchAlgorithmException;
}
