/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.share.net;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.InetAddress;
import java.net.Socket;

/**
 * @author Ian Thompson
 * 
 */
public class Receptor {

	/**
	 * 
	 */
	public final ObjectInputStream in;

	/**
	 * 
	 */
	public final ObjectOutputStream out;

	/* -- Constructor -- */

	/**
	 * @param socket
	 * @param timeout
	 * @throws IOException
	 */
	public Receptor(Socket socket, int timeout) throws IOException {
		this.socket = socket;
		this.socket.setSoTimeout(timeout);
		this.in = new ObjectInputStream(new BufferedInputStream(socket
				.getInputStream()));
		this.out = new ObjectOutputStream(new BufferedOutputStream(socket
				.getOutputStream()));
		this.out.flush();
	}

	/* -- Connection handling methods -- */

	/**
	 * 
	 * 
	 */
	public void close() {
		if (socket != null) {
			try {
				socket.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * 
	 * @return isConnected
	 */
	public boolean isConnected() {
		return socket != null && socket.isConnected();
	}

	/* -- Data transfer methods -- */

	/**
	 * @param object
	 * @throws IOException
	 * 
	 */
	public void send(Serializable object) throws IOException {
		out.writeUnshared(object);
		out.flush();
	}

	/**
	 * 
	 * @return Object
	 * @throws IOException
	 */
	public Object receive() throws IOException {
		try {
			return in.readObject();
		} catch (ClassNotFoundException e) {
			throw new IOException(e);
		}
	}

	/**
	 * Delegates {@link Socket#getInetAddress()}.
	 * 
	 * @return Socket InetAddress
	 */
	public InetAddress getInetAddress() {
		return socket.getInetAddress();
	}

	/* -- Private -- */

	/**
	 * 
	 */
	private Socket socket = null;
}
